'*************************************      
'*  zicog v0.5                       *
'*  An emulation of the Zilog z80    *
'*  (C) 2009 Michael Rychlik         *
'*************************************
'
'v0.0 -  1 Mar 2009 - Original version
'
'v0.1 -  4 Mar 2009 - The big thing here is adding an overlay loader
'                     and starting to implement Z80 string operations as overlays.
'                     A big thank you to Cluso for the overlay loader concept.
'                     See attached overlay_xxx.spin file for his example overlay handling.
'                   - A lot of other changes not worth mentioning as this is still far from working.
'
'v0.2 -  7 Mar 2009 - All 8080 ops are fixed and the T8080.COM CPU diagnostic passes !
'                   - Pretty much all Z80 ops are in place ready for test.
'
'v0.3 - 10 Mar 2009 - Moved A reg and flags into COG.
'
'v0.4 - 16 Mar 2009 - Managed to get the EXZ80DOC Z80 cpu diagnostic to run and pass many tests.
'
'v0.5 - xx xxx 2009 - Moved A reg and flags back into HUB. A couple of percent slower but saves 15 or so LONGs.
'                     Many fixes to get alu ops, logic ops, double adds and neg passing in EXZ80DOC tests.
'                     Moved NEG function (alu_neg) into overlay to save LONGs.

CON

  'Z80 flag bits
  sign_bit   = %10000000
  zero_bit   = %01000000
  aux_bit    = %00010000
  parity_bit = %00000100
  carry_bit  = %00000001

  'I/O control block commands
  io_cmd_out   = $01 
  io_cmd_in    = $02 
  io_cmd_break = $03
  
VAR
  long  cog

PUB start(cpu_params) : okay | reg_base, io_base

'' Start CPU simulator - starts a cog
'' returns false if no cog available
''
''   cpu_par_params is a pointer to CPU parameter list
                                                                                           
  stop

  reg_base := LONG[cpu_params]
  c_reg    := reg_base + 0
  b_reg    := reg_base + 1
  e_reg    := reg_base + 2
  d_reg    := reg_base + 3
  l_reg    := reg_base + 4
  h_reg    := reg_base + 5
  f_reg    := reg_base + 6
  a_reg    := reg_base + 7
  im_reg   := reg_base + 8
  ix_reg   := reg_base + 18
  iy_reg   := reg_base + 20
  sp_reg   := reg_base + 22
  pc_reg   := reg_base + 24

  ram_base    := LONG[cpu_params + 4]
  ram_size    := LONG[cpu_params + 8]
  rom_base    := LONG[cpu_params + 12]
  rom_address := LONG[cpu_params + 16]

  io_base    := LONG[cpu_params + 20]
  io_command := io_base
  io_port    := io_base + 4
  io_data    := io_base + 8

'  cpu_control := LONG[cpu_params + 24]

  dispatch_tab_addr      := @dispatch_table_00
  dispatch_tab_00_addr   := @dispatch_table_00
  dispatch_tab_cb_addr   := @dispatch_table_cb
  dispatch_tab_dd_addr   := @dispatch_table_dd
  dispatch_tab_ddcb_addr := @dispatch_table_ddcb
  dispatch_tab_ed_addr   := @dispatch_table_ed
  
  overlay_tab_addr       := @overlay_table

  'The following code sets up the Overlay Load Parameters
  daa_ovl_     := OverlayParams(@daa_ovl,    @daa_ovl_end)
  ldi_ovl_     := OverlayParams(@ldi_ovl,     @ldi_ovl_end)
  cpi_ovl_     := OverlayParams(@cpi_ovl,     @cpi_ovl_end)
  ini_ovl_     := OverlayParams(@ini_ovl,     @ini_ovl_end)
  outi_ovl_    := OverlayParams(@outi_ovl,    @outi_ovl_end)
  ldd_ovl_     := OverlayParams(@ldd_ovl,     @ldd_ovl_end)
  cpd_ovl_     := OverlayParams(@cpd_ovl,     @cpd_ovl_end)
  ind_ovl_     := OverlayParams(@ind_ovl,     @ind_ovl_end)
  outd_ovl_    := OverlayParams(@outd_ovl,    @outd_ovl_end)
  ldir_ovl_    := OverlayParams(@ldir_ovl,    @ldir_ovl_end)
  cpir_ovl_    := OverlayParams(@cpir_ovl,    @cpir_ovl_end)
  inir_ovl_    := OverlayParams(@inir_ovl,    @inir_ovl_end)
  otir_ovl_    := OverlayParams(@otir_ovl,    @otir_ovl_end)
  lddr_ovl_    := OverlayParams(@lddr_ovl,    @lddr_ovl_end)
  cpdr_ovl_    := OverlayParams(@cpdr_ovl,    @cpdr_ovl_end)
  indr_ovl_    := OverlayParams(@indr_ovl,    @indr_ovl_end)
  otdr_ovl_    := OverlayParams(@otdr_ovl,    @otdr_ovl_end)
  out_ovl_     := OverlayParams(@out_ovl,     @out_ovl_end)
  in_ovl_      := OverlayParams(@in_ovl,      @in_ovl_end)
  alu_neg_ovl_ := OverlayParams(@alu_neg_ovl, @alu_neg_ovl_end)
  break_ovl_ := OverlayParams(@break_ovl,     @break_ovl_end)  
  '....... add more overlays as required.

  'Initilize registers
  pc := WORD[pc_reg]
  flags := WORD[f_reg]
  sp := WORD[sp_reg]
  
  okay := cog := cognew(@entry, 0) + 1                     'Start emulator in a new COG

PUB OverlayParams (o_start, o_end) : params | len, hubend, cogend
'This code sets up the parameters for an overlay (in a format to increase overlay loading speed)
  len := o_end - o_start                                   'length of hub overlay (bytes)
  hubend := o_start + len - 1                              'hub END address (last long) +3 (speed quirk in overlay loader)
  cogend := ((@OVERLAY_START - @entry + len) / 4) - 1      'cog END address (last long)
  params := hubend << 16 + cogend                          'set parameters


PUB stop

'' Stop CPU simulator - frees a cog

  if cog
    cogstop(cog~ - 1)


DAT
'---------------------------------------------------------------------------------------------------------

                        org     0

'---------------------------------------------------------------------------------------------------------
entry
OVERLAY_LOAD_RET                                            
                        jmp       #RESIDENT_CODE           'Go execute resident  code first (loaded automatically at CogNew)
                                                           'Can use as a RET instruction AFTER execution of the loaded overlay        
'---------------------------------------------------------------------------------------------------------
'RESIDENT ROUTINES follow...
'
RESIDENT_CODE 
                        mov     dira, heartbeat_pin        'Set Pins to output

'---------------------------------------------------------------------------------------------------------
'Instruction fetch loop starts here
not_implemented
undocumented
halt
fetch
                        'mov     OVERLAY_PAR, break_ovl_    '\\ copy break_ovl's parameters                                 
                        'call    #OVERLAY_LOAD              '// load & execute break_ovl                  
                        
                        xor     outa, heartbeat_pin        'N.B. If using heartbeat for KIPS measurement
                                                           'remember to multiply the frequency by 2.                         
                        mov     dispatch_tab_addr, dispatch_tab_00_addr

dispatch                mov     address, pc
                        call    #read_memory_byte          'Read the instruction byte there 
                        add     pc, #1                     'Increment the program counter

dispatch_2              shl     data_8, #2                 'Mul by 4 to get a LONG offset into the table
                        add     data_8, dispatch_tab_addr  'Index into table
                        rdlong  vector, data_8             'Get vector from table
vect_1                  jmp     vector                     'Jump to the instruction handler

vect_2
vect_3                  ror     vector, #9                 'shift to 2nd (or 3rd) vector
                        jmp     vector 


pc                      long    0                          'Z80's Pogram Conter
sp                      long    0                          'Z80's Stack pointer
flags                   long    0                          'Z80's flags
xy_reg                  long    0                          'Points to IX or IY depending on DD or FD prefix
'cpu_control             long    0
heartbeat_pin           long    %00000000_00000001_00000000_00000000
dispatch_tab_addr       long    0
dispatch_tab_00_addr    long    0-0
dispatch_tab_cb_addr    long    0-0
dispatch_tab_dd_addr    long    0-0
dispatch_tab_ddcb_addr  long    0-0
dispatch_tab_ed_addr    long    0-0
vector                  long    0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Source Functions

get_b                   rdbyte  data_8, b_reg
                        jmp     #vect_2

get_c                   rdbyte  data_8, c_reg
                        jmp     #vect_2

get_d                   rdbyte  data_8, d_reg
                        jmp     #vect_2

get_e                   rdbyte  data_8, e_reg
                        jmp     #vect_2

get_h                   rdbyte  data_8, h_reg
                        jmp     #vect_2

get_l                   rdbyte  data_8, l_reg
                        jmp     #vect_2

get_a                   rdbyte  data_8, a_reg
                        jmp     #vect_2

get_bc                  rdword  data_16, b_reg
                        jmp     #vect_2
                        
get_de                  rdword  data_16, d_reg
                        jmp     #vect_2

get_hl                  rdword  data_16, h_reg
                        jmp     #vect_2

get_af                  rdbyte  data_16, a_reg
                        shl     data_16, #8
                        or      data_16, flags
                        jmp     #vect_2

get_ixy                 rdword  data_16, xy_reg
                        jmp     #vect_2

get_sp                  mov     data_16, sp
                        jmp     #vect_2

get_pc                  mov     data_16, pc
                        jmp     #vect_2

get_m
get_memory_hl           rdword  address, h_reg
get_memory_addr         call    #read_memory_byte
                        jmp     #vect_2

get_memory_bc           rdword  address, b_reg
                        jmp     #get_memory_addr

get_memory_de           rdword  address, d_reg
                        jmp     #get_memory_addr


address_frm_xy_disp     'FIXME This must be switchable to get iy as well
                        mov     address, pc                'Get displacement from immediate byte
                        call    #read_memory_byte
                        add     pc, #1
                        shl     data_8, #24                'Sign extent the displacement
                        sar     data_8, #24
                        rdword  address, xy_reg            'Get the address from IX or IY index register
                        shl     address, #16               'Sign extend the address
                        sar     address, #16
                        adds    address, data_8            'Twos complement addition of address and displacement
address_frm_xy_disp_ret ret

get_mem_xy_disp         call    #address_frm_xy_disp
                        jmp     #get_memory_addr

address_stash           long    0

get_stash               mov     address, address_stash
                        mov     data_8, data_stash
                        jmp     #vect_2


get_immediate_byte      mov     address, pc                'Read a BYTE from location given by "pc"
                        add     pc, #1                     'of Z80 RAM into byte data_8 and increment pc
                        jmp     #get_memory_addr

 
get_immediate_word      mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of Z80 RAM into byte data_16 and pc = pc + 2
                        add     pc, #2
                        jmp     #vect_2

get_tos                 mov     address, sp
                        call    #read_memory_word
                        jmp     #vect_2


test_flag_true          mov     alu, vector
                        shr     alu, #18
                        test    flags,alu wz
                        jmp     #vect_2
                        

test_flag_false         mov     alu, vector
                        shr     alu, #18
                        xor     flags, #$FF
                        test    flags, alu wz
                        xor     flags, #$FF
                        jmp     #vect_2


return_conditional
              if_nz     jmp     #return_unconditional
                        jmp     #fetch

return_unconditional    mov     address, sp                'Get return address from the stack
                        call    #read_memory_word
                        mov     pc, data_16                'Jump to it
                        add     sp, #2                     'Complete the "pop"
                        jmp     #fetch

jump_rel_conditional
              if_nz     jmp     #jump_rel_unconditional
                        add     pc, #1
                        jmp     #fetch

jump_rel_unconditional  mov    address, pc                 'Get target displacement from immediate byte
                        call   #read_memory_byte
                        shl    data_8, #24                 'Offset is two's comp. so sign extend it.
                        sar    data_8, #24
                        add    pc, data_8                  'and add to the PC.
                        jmp    #fetch
                        
jump_conditional        
              if_nz     jmp     #jump_unconditional
                        add     pc, #2
                        jmp     #fetch
              

jump_unconditional      mov     address, pc                'Get target address from immediate word
                        call    #read_memory_word
                        mov     pc, data_16
                        jmp     #fetch                     'and jump to it

call_conditional
              if_nz     jmp     #call_unconditional
                        add     pc, #2
                        jmp     #fetch

call_unconditional      mov     data_16, pc
                        add     data_16, #2
                        call    #push
                        jmp     #jump_unconditional

dec_jmp_nz              rdbyte  data_8, b_reg              'Decrement B register
                        sub     data_8, #1 wz
                        wrbyte  data_8, b_reg
              if_nz     jmp     jump_rel_unconditional     'Jump if not zero
                        add     pc, #1                     'Else skip immediate byte
                        jmp     #fetch                        
        

restart                 mov     data_16, pc                'Push return aaddress
                        call    #push
                        shr     vector, #9                 'Get restart target address from vector
                        mov     pc, vector                 'and jump there.
                        jmp     #fetch
                        

push                    sub     sp, #2
                        mov     address, sp
                        call    #write_memory_word
push_ret                ret


push_                   call    #push
                        jmp     #fetch


pop                     mov     address, sp
                        call    #read_memory_word
                        add     sp, #2                     'Complete the "pop"
pop_ret                 ret


pop_                    call    #pop
                        jmp     #vect_2


z80_CB                  mov     dispatch_tab_addr, dispatch_tab_cb_addr  'Z80, CB Prefix
                        jmp     #dispatch 


z80_DD                  mov     xy_reg, ix_reg             'Point XY reg to IX reg for DD prefix ops
                        mov     dispatch_tab_addr, dispatch_tab_dd_addr  'Z80, DD Prefix
                        jmp     #dispatch 


z80_DDCB                mov     data_stash, data_8         'Z80 DDCB prefix
                        mov     address_stash, address
                        mov     address, pc
                        call    #read_memory_byte          'Read the instruction byte there 
                        add     pc, #1                     'Increment the program counter
                        shr     data_8, #3                 'DDCB table only has 32 entries so discard 3 low bits 
                        mov     dispatch_tab_addr, dispatch_tab_ddcb_addr
                        jmp     #dispatch_2 

data_stash              long    0

z80_ED                  mov     address, pc                'Z80, ED Prefix
                        call    #read_memory_byte          'Read the instruction byte there 
                        add     pc, #1                     'Increment the program counter
                        sub     data_8, #$40 wc            'ED dispatch table starts at $40  FIXME Should we range check ED opcodes?
                        mov     dispatch_tab_addr, dispatch_tab_ed_addr
                        jmp     #dispatch_2 


z80_FD                  'FIXME Set something here to select IY
                        mov     xy_reg, iy_reg             'Point XY reg to IY reg for FD prefix ops
                        mov     dispatch_tab_addr, dispatch_tab_dd_addr 'Z80, FD Prefix
                        jmp     #dispatch 


store_a_direct          mov     address, data_16
                        rdbyte   data_8, a_reg
                        call    #write_memory_byte
                        jmp     #fetch


load_a_direct           mov     address, data_16
                        call    #read_memory_byte
                        wrbyte  data_8, a_reg     
                        jmp     #fetch


store_word_direct       mov     alu, data_16               'Save the word for a while
                        mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of Z80 RAM into byte data_16 and pc = pc + 2
                        add     pc, #2
                        mov     address, data_16           'Use the immediate word as target address
                        mov     data_16, alu               'Get the word back again                        
                        call    #write_memory_word         'and store it
                        jmp     #fetch


load_word_direct        mov     address, pc                'Read a WORD from location given by "pc"
                        call    #read_memory_word          'of Z80 RAM into byte data_16 and pc = pc + 2
                        add     pc, #2
                        mov     address, data_16           'Use the immediate word as target address
                        call    #read_memory_word          'and load the word.
                        jmp     #vect_2

z80_exx
in_c
out_c

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Data Destination Functions

put_b                   wrbyte  data_8, b_reg
                        jmp     #fetch

put_c                   wrbyte  data_8, c_reg
                        jmp     #fetch

put_d                   wrbyte  data_8, d_reg
                        jmp     #fetch

put_e                   wrbyte  data_8, e_reg
                        jmp     #fetch

put_h                   wrbyte  data_8, h_reg
                        jmp     #fetch

put_l                   wrbyte  data_8, l_reg
                        jmp     #fetch

put_a                   wrbyte  data_8, a_reg   
                        jmp     #fetch
                        
put_bc                  wrword  data_16, b_reg
                        jmp     #fetch

put_de                  wrword  data_16, d_reg
                        jmp     #fetch

put_hl                  wrword  data_16, h_reg
                        jmp     #fetch

put_af                  mov     flags, data_16
                        and     flags, #$FF
                        shr     data_16, #8
                        wrbyte  data_16, a_reg
                        jmp     #fetch

put_ixy                 wrword  data_16, xy_reg
                        jmp     #fetch

put_sp                  mov     sp, data_16
                        jmp     #fetch

put_pc                  mov     pc, data_16
                        jmp     #fetch

put_m                 
put_memory_hl           rdword  address, h_reg
put_memory_addr         call    #write_memory_byte
                        jmp     #fetch

put_memory_bc           rdword  address, b_reg
                        jmp     #put_memory_addr

put_memory_de           rdword  address, d_reg
                        jmp     #put_memory_addr

put_mem_xy_disp         mov     alu, data_8                'Save data_8 from corruption
                        call    #address_frm_xy_disp
                        mov     data_8, alu                'Get data_8 back again
                        jmp     #put_memory_addr

'FIXME is this the best way to do this ? This instruction does not fit our vector frame work.
ld_ix_disp_byte         call    #address_frm_xy_disp       'Make an address out of index reg and displacement
                        mov     address_stash, address     'Stash the address away for a while
                        mov     address, pc                'Get the immediate data byte into data_8
                        call    #read_memory_byte
                        add     pc, #1
                        
put_memory_sa           mov     address, address_stash     'Write data_8 out to stashed address 
                        jmp     #put_memory_addr

put_tos                 mov     address, sp
                        call    #write_memory_word
                        jmp     #fetch


'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Memory Access Functions

'Read a BYTE from location given by "address" of Z80 RAM into byte data_8
read_memory_byte        cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_ae     jmp     #read_rom
                        mov     hub_pointer, ram_base                              'Point to the RAM
                        add     hub_pointer, address       'Add address offset
                        rdbyte  data_8, hub_pointer        'Read the byte from there
read_memory_byte_ret    ret


read_rom                mov     rom_address, address
                        sub     rom_address, rom_start     'address becomes offset into ROM from start of ROM                  
                        mov     hub_pointer, rom_base      'Point to the ROM
                        add     hub_pointer, rom_address   'Add address offset                     
                        rdbyte  data_8, hub_pointer        'Read the byte from there
                        jmp     read_memory_byte_ret


'Write the BYTE data_8 to location given by "address"
write_memory_byte       cmp     address, ram_size wc,wz    'Is the given address within the RAM range ?
              if_b      mov     hub_pointer, ram_base      'Point to the RAM
              if_b      add     hub_pointer, address       'Add address offset
              if_b      wrbyte  data_8, hub_pointer        'Write the byte there
write_memory_byte_ret   ret


'Read a WORD from location given by "address" of Z80 RAM into byte data_16
read_memory_word        call    #read_memory_byte          'Read the low byte
                        mov     data_16, data_8
                        ror     data_16, #8
                        add     address, #1                            
                        call    #read_memory_byte          'Read the high byte
                        or      data_16, data_8
                        rol     data_16, #8
read_memory_word_ret    ret

'Write the WORD data_16 to location given by "address"
write_memory_word       mov     data_8, data_16
                        call    #write_memory_byte         'Write the LOW byte
                        shr     data_8, #8
                        add     address, #1
                        call    #write_memory_byte
write_memory_word_ret   ret

'The following configuration "constants" set from COG PARameters.
ram_base                long    0-0                        'HUB address of the Z80 RAM area
ram_size                long    0-0                        'Length in bytes of available Z80 RAM
rom_base                long    0-0                        'HUB address of the BOOT ROM image            
rom_start               long    0-0                        'ROM starts at this address in Z80 space
rom_address             long    0-0

'Z80 RAM read/write vars.
hub_pointer             long    ram_base                   'Pointer into HUB RAM
address                 long    0                          'Address param for read/write funcs
data_8                  long    0                          'Data bytes read into here
data_16                 long    0                          'Data words read into here
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'Z80 Operation Functions

set_carry               or      flags, #carry_bit          'Set Carry Flag
                        andn    flags, #(aux_bit | %00000010)
                        jmp     #fetch


complement_carry        xor     flags, #carry_bit          'Complement Carry Flag
                        andn    flags, #%00000010
                        jmp     #fetch


complement_byte         xor     data_8, #$FF
                        or      flags, #(aux_bit | %00000010)
                        jmp     #vect_3

increment_byte
                        add     data_8, #1                 'Do the increment

                        andn    flags, #$FE                'Clear the flags to be set here

                        test    data_8, #$0F wz            'Set AUX flag when low nibble is zero
              if_z      or      flags, #aux_bit 

                        cmp     data_8, #$80 wz
              if_z      or      flags, #parity_bit         'Set overflow flag when sign becomes negative


incdec_zero_sign        test    data_8, #$FF wz
              if_z      or      flags, #zero_bit           'Set zero flag when result is zero

                        test    data_8, #128 wz            'Set sign flag from top bit.
                        muxnz   flags, #sign_bit           'NOTE: use code above for more accurate flag setting

                        jmp     #vect_3


decrement_byte          sub     data_8, #1                 'Do the increment
                        
                        andn    flags, #$FE                'Clear the flags to be set here
                        or      flags, #%00000010

                        mov     alu, data_8               
                        and     alu, #$0F
                        cmp     alu, #$0F wz
             if_z       or      flags, #aux_bit            'Set AUX flag when low nibble is $0F.

                        cmp     data_8, #$7F wz
             if_z       or      flags, #parity_bit         'Set overflow flag when sign becomes positive

                        jmp     #incdec_zero_sign          'Set zero and sign flags as per increment.


alu_add                 rdbyte  alu, a_reg                 'Load A
                        mov     data_16, alu
                        add     data_16, data_8            'Result is in sum
                        mov     flags, #$00                'Clear all flags

fix_flags               mov     aux, alu
                        xor     aux, data_8
                        xor     aux, data_16

                        test    aux, #aux_bit wc
                        muxc    flags, #aux_bit

                        test    aux, #%1_0000_0000 wc
                        muxc    flags, #parity_bit
                        test    aux, #%1000_0000 wc
              if_c      xor     flags, #parity_bit

                        test    aux, #$100 wz 
                        muxnz   flags, #carry_bit

                        mov     data_8, data_16            'Post result and
                        jmp     #incdec_zero_sign          'sort out zero and sign flags


alu_add_with_carry      rdbyte  alu, a_reg                 'Load A

                        test    flags, #carry_bit wc       'Set prop's carry from Z80 carry flag
                        mov     data_16, alu
                        addx    data_16, data_8            'Do the op
                        mov     flags, #$00                'Clear all flags
                        jmp     #fix_flags


alu_sub                 rdbyte  alu, a_reg                 'Load A
                        mov     data_16, alu 
                        sub     data_16, data_8            'Do the op
                        mov     flags, #%00000010          'Clear all flags excepy N
                        jmp     #fix_flags


alu_sub_with_borrow     rdbyte  alu, a_reg                 'Load A
                        test    flags, #carry_bit wc       'Set prop's carry from Z80 carry flag
                        mov     data_16, alu 
                        subx    data_16, data_8            'Do the op
                        mov     flags, #%00000010          'Clear all flags excepy N
                        jmp     #fix_flags


zsp_flags               and     alu, #$FF wz, wc           'Set zero, sign and parity flags from alu content
                        muxz    flags, #zero_bit           'Set Z80 zero flag from props zero
                        muxnc   flags, #parity_bit         'That "and" above gives parity in props carry flag
                        test    alu, #128 wz               'Top bit is sign
                        muxnz   flags, #sign_bit
                        mov     data_8, alu
                        jmp     #vect_3


alu_and                 rdbyte  alu, a_reg                 'Load A
                        and     alu, data_8 wz, wc         'Do the AND and get flags
                        or      flags, #aux_bit            'AND always sets AUX flag (8085) 

alu_logic_flags         andn    flags, #(carry_bit | %00000010) 'Logic ops always clear carry and N flags
                        jmp     #zsp_flags


alu_xor                 rdbyte  alu, a_reg                 'Load A
                        xor     alu, data_8 wz, wc         'Do the XOR and get flags
                        andn    flags, #aux_bit            'XOR always clears AUX flag
                        jmp     #alu_logic_flags


alu_or                  rdbyte  alu, a_reg                 'Load A
                        or      alu, data_8 wz, wc         'Do the OR and get flags
                        andn    flags, #aux_bit            'OR always clears AUX flag
                        jmp     #alu_logic_flags


alu_rlc                 shl     data_8, #1                 'Shift accumulator left by one
                        test    data_8, #%1_0000_0000 wz   'Move any 8 bit carry out bit into acc LSB
                        muxnz   data_8, #1
rot_end_1               muxnz   flags, #carry_bit          'Set carry flag from 8 bit overflow
                        andn    flags, #(aux_bit | %00000010) 
                        jmp     #vect_3

  
alu_rrc                 shr     data_8, #1 wc              'Shift accumulator right by one
                        muxc    data_8, #%10000000         'Move props carry to acc. MSB
rot_end_2               muxc    flags, #carry_bit          'Set carry flag from props carry
                        andn    flags, #(aux_bit | %00000010) 
                        jmp     #vect_3


alu_ral                 shl     data_8, #1                 'Shift accumulator left by one
                        test    flags, #carry_bit wz       'Set acc. LSB from carry flag
                        muxnz   data_8, #1                        
                        test    data_8, #%1_0000_0000 wz   'Set carry flag from 8 bit overflow
                        jmp     #rot_end_1


alu_rar                 shr     data_8, #1 wc              'Shift accumulator right by one
                        test    flags, #carry_bit wz       'Set acc. MSB from carry flag
                        muxnz   data_8, #%10000000
                        jmp     #rot_end_2


alu_sla                ' shl     data_8, #1                 'FIXME shifts must set more flags !!!
                       ' test    data_8, #%1_0000_0000 wc   'Set props carry flag from 8 bit overflow
                       ' muxc    af, #carry_bit             'Set carry_bit from props cary
                       ' jmp     #vect_3


alu_sra                ' test    data_8, #%1000_0000 wz     'Set props zero from sign bit
                       ' shr     data_8, #1 wc              'Shift and get bit 0 to props carry
                       ' muxc    af, #carry_bit             'Set carry_bit from props cary
                       ' muxnz   data_8, #%1000_0000        'Set sign bit from props zero
                       ' jmp     #vect_3


alu_srl                ' shr     data_8, #1 wc              'Shift and get bit 0 to props carry
                       ' muxc    af, #carry_bit             'Set carry_bit from props cary
                       ' andn    data_8, #%1000_0000        'Ensure top bit of result is clear.                 
                       ' jmp     #vect_3


alu_bit              '   mov     alu, vector                'Make bit select mask from bit pos. in vector
                     '   shr     alu, #18
                     '   mov     aux, #1
                     '   shl     aux, alu                        

                     '   test    data_8, aux wz             'Selected bit set ?
                     '   muxz    aux, #zero_bit             'Set Z80 zero flag from props zero
                       ' jmp     #fetch                     'No result to write so bail out fast.


alu_res              '   mov     alu, vector                'Make bit select mask from bit pos. in vector
                     '   shr     alu, #18
                     '   mov     aux, #1
                     '   shl     aux, alu                        

                     '   andn    data_8, aux                'Clear the selected bit
                        'jmp     #vect_3

alu_set              '   mov     alu, vector                'Make bit select mask from bit pos. in vector
                     '   shr     alu, #18
                     '   mov     aux, #1
                     '   shl     aux, alu                        

                     '   or      data_8, aux                'Set the selected bit
                        jmp     #vect_3

{
                case 0x44:                      /* NEG */
                        temp = hreg(AF);
                        AF = (-(AF & 0xff00) & 0xff00);
                        AF |= ((AF >> 8) & 0xa8)           Set the sign bit S from result
                        AF |= (((AF & 0xff00) == 0) << 6)  Set the zero bit Z from result

                        AF |= (((temp & 0x0f) != 0) << 4)  Set the AUX bit
                        AF |= ((temp == 0x80) << 2)        Set the Overflow bit V
                        AF |= 2                            Set the N bit
                        AF |= (temp != 0);                 Set the carry bit C
}




increment_word          add     data_16, #1
                        jmp     #vect_3


decrement_word          sub     data_16, #1
                        jmp     #vect_3


double_add_hl_cy        'FIXME for Z80
                        'FIXME Should set most flags !


double_add_hl           rdword  alu, h_reg
                        
double_add              mov     aux, alu                   'Take a copy of alu before operation for AUX carry determination

                        add     alu, data_16
                        test    alu, mask_bit_16 wz
                        muxnz   flags, #carry_bit
                        andn    flags, #%00000010

                        xor     aux, data_16
                        xor     aux,alu                    'Determine the auxillary carry flag
                        test    aux, aux_bit_shifted wc
                        muxc    flags, #aux_bit

                        mov     data_16, alu
                        jmp     #vect_3

aux_bit_shifted         long    (aux_bit << 8)

double_add_ixy          rdword  alu, xy_reg
                        jmp     #double_add


double_sub_hl_cy        'FIXME for Z80
                        'FIXME Should set most flags !


exchange                rdword address, h_reg
                        mov    data_8, data_16
                        mov    data_16, address
                        wrword  data_8, h_reg
                        jmp    #vect_3     


vector_overlay          mov     ovl, vector                'Extract overlay param number from vector bits 2 to 26
                        shr     ovl, #7
                        add     ovl, overlay_tab_addr      'Offset into overlay parameter table
                        rdlong  OVERLAY_PAR, ovl           'Get this overlays parameters
                        jmp     #OVERLAY_LOAD              'load it and run it.              

ovl                     long      0
overlay_tab_addr        long      0-0

'---------------------------------------------------------------------------------------------------------
'Z80 String operation parts

'---------------------------------------------------------------------------------------------------------
'Misc variables
alu                     long    0
aux                     long    0
nibble                  long    0
mask_bit_16             long    $0001_0000

'---------------------------------------------------------------------------------------------------------
'Pointers to Z80 registers in HUB RAM
b_reg                   long    0
c_reg                   long    0
d_reg                   long    0
e_reg                   long    0
h_reg                   long    0
l_reg                   long    0
f_reg                   long    0
a_reg                   long    0
im_reg                  long    0
ix_reg                  long    0
iy_reg                  long    0
sp_reg                  long    0
pc_reg                  long    0

'---------------------------------------------------------------------------------------------------------
'The I/O control block
io_command              long    0
io_port                 long    0
io_data                 long    0
'---------------------------------------------------------------------------------------------------------
                
'---------------------------------------------------------------------------------------------------------
' OVERLAY LOADER follows...
' An even number of longs will be loaded (for efficiency)
' Flags c & z will be maintained
'

Overlay_par             long    0-0                        'overlay parameters for the OVERLAY_LOAD

break_ovl_              long    0-0                                                                                                                               

_0x400                  long    $0000_0400                 'inc/decrement destination by 2
_djnz0                  djnz    overlay_par, #overlay_copy2 'prototype instruction (moved to overlay instruction)
t1                      long    0                          'used to determine if overlay already loaded (can be used by other code)

OVERLAY_LOAD            mov     OVERLAY_START, _djnz0      'Copy djnz instruction to head of overlay area.
                        movd    overlay_copy2, overlay_par 'move cog END address into rdlong instruction
                        sub     overlay_par, #1            'decrement cog End address by 1
                        movd    overlay_copy1, overlay_par 'move cog END-1 address into rdlong instruction
                        shr     overlay_par, #16           'extract the overlay## hub END address (remove cog address)
overlay_copy2           rdlong  0-0, overlay_par           'copy long from hub to cog   (hptr ignores last 2 bits!)
                        sub     overlay_par, #7            'decrement hub ptr by 1 long (prev by 1, now by 7)
                        sub     overlay_copy2, _0x400      'decrement cog (destination) address by 2
overlay_copy1           rdlong  0-0, overlay_par           'copy long from hub to cog
                        sub     overlay_copy1, _0x400      'decrement cog (destination) address by 2 
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
OVERLAY_START '<--- djnz  overlay_par,#overlay_copy2    'decrement hub ptr by 1 long (now by 1, next by 7)
              '^^^ The above instruction is moved in by the loader and is overwritten by the first overlay
              '    instruction during the final overlay load (rdlong) instruction. (loading is done in reverse)     
'---------------------------------------------------------------------------------------------------------
                        fit       $1F0
'---------------------------------------------------------------------------------------------------------
              
'---------------------------------------------------------------------------------------------------------
'daa overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
daa_ovl
                        rdbyte  data_8, a_reg
                        mov     nibble, data_8             'Isolate low nibble of accumulator
                        and     nibble, #$0F                        
                        cmp     nibble, #10 wc             'Test for greater than 9                
                        test    flags, #aux_bit wz            'Test auxillary carry
            if_nz_or_nc add     data_8, #$06               'Add 6 to accumulator if greater than 9 or AUX carry
                        add     nibble, #6                 'Check for AUX carry from low nibble
                        test    nibble, #%00010000 wz
                        muxnz   flags, #aux_bit            'and set AUX flag if so.      
                        mov     nibble, data_8             'Isolate high nibble of accumulator
                        shr     nibble, #4
                        cmp     nibble, #10 wc             'Test for greater than 9
                        test    flags, #carry_bit wz       'Test carry flag
            if_nz_or_nc add     data_8, #$60               'Add (6 << 4) to accumulator if greater than 9 or carry
                        test    data_8, #$100 wz           'Check for 8 bit carry out
            if_nz       or      flags, #carry_bit          'Set carry flag if so (N.B. Do NOT clear carry if not so)
                        and     data_8, #$FF wz, wc
                        muxz    flags, #zero_bit           'Set Z80 zero flag from props zero
                        muxnc   flags, #parity_bit
                        test    data_8, #128 wz
                        muxnz   flags, #sign_bit
                        wrbyte  data_8, a_reg                        
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)

daa_ovl_end 
                        fit    $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ldi overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
ldi_ovl                 rdword  address, h_reg
                        call    #read_memory_byte
                        add     address, #1
                        wrword  address, h_reg
                        rdword  address, d_reg
                        call    #write_memory_byte
                        add     address, #1
                        wrword  address, d_reg
                        rdword  data_16, b_reg
                        sub     data_16, #1 wz
                        muxnz   flags, #parity_bit
                        wrword  data_16, b_reg 
                        andn    flags, #(aux_bit  | %00010010)
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
ldi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpi overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
cpi_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
cpi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ini overlay. This is a "load/execute" overlay.
' (HL) <- [C];
' B <- B-1;
' HL <- HL+1
                        org     OVERLAY_START
ini_ovl                 jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
ini_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'outi overlay. This is a "load/execute" overlay.
'[C] <- (HL);
'B <- B-1;
'HL <- HL+1;
                        org OVERLAY_START
outi_ovl                rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port            
                        rdword  address, h_reg             'Set port data from memory at HL
                        call    #read_memory_byte
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
                        wrbyte  data_8, io_command         
:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
              if_nz     jmp     :wait
                        rdbyte  data_8, b_reg              'Update B reg
                        sub     data_8, #1
                        wrbyte  data_8, b_reg
                        add     address, #1                'Update HL reg
                        wrword  address, h_reg
                        jmp     #fetch                        
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
outi_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ldd overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
ldd_ovl                 rdword  address, h_reg
                        call    #read_memory_byte
                        sub     address, #1
                        wrword  address, h_reg
                        rdword  address, d_reg
                        call    #write_memory_byte
                        sub     address, #1
                        wrword  address, d_reg
                        rdword  data_16, b_reg
                        sub     data_16, #1 wz
                        muxnz   flags, #parity_bit
                        wrword  data_16, b_reg
                        andn    flags, #(aux_bit  | %00010010)
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
ldd_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpd overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
cpd_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
cpd_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ind overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
ind_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
ind_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'outd overlay. This is a "load/execute" overlay.
' [C] <- (HL);
' B <- B-1;
' HL <- HL-1;
                        org     OVERLAY_START
outd_ovl                rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port            
                        rdword  address, h_reg             'Set port data from memory at HL
                        call    #read_memory_byte
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
                        wrbyte  data_8, io_command         
:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
               if_nz    jmp     :wait
                        rdbyte  data_8, b_reg              'Update B reg
                        sub     data_8, #1
                        wrbyte  data_8, b_reg
                        sub     address, #1                'Update HL reg
                        wrword  address, h_reg
                        jmp     #fetch                        
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
outd_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'ldir overlay. This is a "load/execute" overlay.
' (DE) <- (HL);
' HL   <- HL+1;
' DE   <- DE+1;
' BC   <- BC-1;
' repeat until BC = 0
                        org     OVERLAY_START
ldir_ovl                rdword  :hl, h_reg
                        rdword  :de, d_reg
                        rdword  :bc, b_reg

:loop                   mov     address, :hl               'Move mem [HL] to mem [de]
                        call    #read_memory_byte
                        mov     address, :de
                        call    #write_memory_byte
                        add     :hl, #1                    'Advance the pointers
                        add     :de, #1
                        djnz    :bc, #:loop                'Decrement counter and maybe loop

                        wrword  :hl, h_reg                 'Update the registers
                        wrword  :de, d_reg
                        wrword  :bc, b_reg
                        andn    flags, #(parity_bit | aux_bit | %00010010)
                        jmp     #fetch

:hl                     long    0
:de                     long    0
:bc                     long    0
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
ldir_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpir overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
cpir_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
cpir_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'inir overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
inir_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
inir_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'otir overlay. This is a "load/execute" overlay.
' [C] <- (HL);
' B <- B-1;
' HL <- HL+1;
                        org     OVERLAY_START
otir_ovl                rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port
                        rdword  address, h_reg
                        rdbyte  :b, b_reg 
:loop                   call    #read_memory_byte          'Set port data from memory at HL
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
                        wrbyte  data_8, io_command         

:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
               if_nz    jmp     :wait

                        add     address, #1                'Advance address pointer
                        djnz    :b, #:loop                 'Decrement counter and maybe loop
                        wrbyte  :b, b_reg                  'Update counter B reg
                        wrword  address, h_reg             'Update HL reg to next address
                        jmp     #fetch                        

:b                      long    0
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
otir_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'lddr overlay. This is a "load/execute" overlay.
' (DE) <- (HL);
' HL <- HL-1;
' DE <- DE-1;
' BC <- BC-1;
' repeat until BC = 0
                        org     OVERLAY_START
lddr_ovl                rdword  :hl, h_reg
                        rdword  :de, d_reg
                        rdword  :bc, b_reg

:loop                   mov     address, :hl               'Move mem [HL] to mem [de]
                        call    #read_memory_byte
                        mov     address, :de
                        call    #write_memory_byte
                        sub     :hl, #1                    'Advance the pointers
                        sub     :de, #1
                        djnz    :bc, #:loop                'Decrement counter and maybe loop

                        wrword  :hl, h_reg                 'Update the registers
                        wrword  :de, d_reg
                        wrword  :bc, b_reg
                        andn    flags, #(parity_bit | aux_bit | %00010010)
                        jmp     #fetch

:hl                     long    0
:de                     long    0
:bc                     long    0
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
lddr_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'cpdr overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
cpdr_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
cpdr_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'indr overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
indr_ovl
                        jmp     #fetch
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
indr_ovl_end
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'otdr overlay. This is a "load/execute" overlay.
' [C] <- (HL);
' B <- B-1;
' HL <- HL-1;
                        org     OVERLAY_START
otdr_ovl                rdbyte  data_8, c_reg              'Set the IO port from C
                        wrbyte  data_8, io_port
                        rdword  address, h_reg
                        rdbyte  :b, b_reg
:loop                   call    #read_memory_byte          'Set port data from memory at HL
                        wrbyte  data_8, io_data               
                        mov     data_8, #io_cmd_out        'Set I/O command to OUT...
                        wrbyte  data_8, io_command         

:wait                   rdbyte  data_8, io_command wz      '...and wait for it to be completed.
               if_nz    jmp     :wait

                        sub     address, #1                'Retard (?) address pointer
                        djnz    :b, #:loop                 'Decrement counter and maybe loop                        
                        wrbyte  :b, b_reg                  'Update counter B reg
                        wrword  address, h_reg             'Update HL reg to next address
                        jmp     #fetch
:b                      long    0                        
                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
otdr_ovl_end
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'out overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
out_ovl                 mov     address, pc                'Set IO port no. from immediate byte
                        call    #read_memory_byte
                        add     pc, #1
                        wrbyte  data_8, io_port             

                        rdbyte  data_8, a_reg
                        wrbyte  data_8, io_data               

                        mov     data_8, #io_cmd_out        'Set I/O command to OUT
                        wrbyte  data_8, io_command         

:wait                   rdbyte  data_8, io_command wz      'Wait for the output done
              if_nz     jmp     #:wait

                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
out_ovl_end
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'in overlay. This is a "load/execute" overlay.
                        org     OVERLAY_START
in_ovl                  mov     address, pc                'Set IO port no. from immediate byte
                        call    #read_memory_byte
                        add     pc, #1
                        wrbyte  data_8, io_port
                        mov     data_8, #io_cmd_in         'Set I/O command to IN
                        wrbyte  data_8, io_command

:wait                   rdbyte  data_8, io_command wz      'Wait for the input data  
              if_nz     jmp     #:wait

                        rdbyte  data_8, io_data            'Read the input data to accumulator   
                        wrbyte  data_8, a_reg
                        jmp     #fetch

                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
in_ovl_end
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'alu_neg overlay. This is a "called" overlay.
                        org     OVERLAY_START
alu_neg_ovl
                        rdbyte  data_8, a_reg

                        mov     flags, #%00000010          'Clear all flags except N

                        test    data_8, #$0F wz            
              if_nz     or      flags, #aux_bit

                        cmp     data_8, #$80 wz
              if_z      or      flags, #parity_bit

                        cmp     data_8, #0 wz
              if_nz     or      flags, #carry_bit                        
              
                        neg     data_8, data_8              'Do the negation

                        wrbyte  data_8, a_reg

                        jmp     #incdec_zero_sign

                        long    $0[($ - OVERLAY_START) // 2]    'fill to even number of longs (REQUIRED)
alu_neg_ovl_end

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'break overlay. This is a "called" overlay.
                        org     OVERLAY_START
break_ovl               wrword  pc, pc_reg
                        wrbyte  flags, f_reg
                        wrword  sp, sp_reg                         
'                        mov     data_16, #0                   
                  '      wrword  data_16, cpu_control
':break                  rdword  data_16, cpu_control
'                        cmp     data_16, #0 wz
'              if_z      jmp     #:break

                        mov     data_8, #io_cmd_break      'Set I/O command to BREAK
                        wrbyte  data_8, io_command
:wait                   rdbyte  data_8, io_command wz
              if_nz     jmp     #:wait


                        jmp     OVERLAY_LOAD_RET           'return (indirect) to calling routine

                        long    $0[($ - OVERLAY_START) // 2] 'fill to even number of longs (REQUIRED)
break_ovl_end 
                        fit     $1F0
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
CON
'Overlay numbers.
'Note: These overlay numbers must be the in the order of the overlay param
'      entries in the overlay_table.
daa_ovl_no     = $00
ldi_ovl_no     = $01
cpi_ovl_no     = $02
ini_ovl_no     = $03
outi_ovl_no    = $04
ldd_ovl_no     = $05
cpd_ovl_no     = $06
ind_ovl_no     = $07
outd_ovl_no    = $08
ldir_ovl_no    = $09
cpir_ovl_no    = $0A
inir_ovl_no    = $0B
otir_ovl_no    = $0C
lddr_ovl_no    = $0D
cpdr_ovl_no    = $0E
indr_ovl_no    = $0F
otdr_ovl_no    = $10
out_ovl_no     = $11
in_ovl_no      = $12
alu_neg_ovl_no = $13
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
DAT
' Table of overlay parameters.
' Note: The overlay numbers defined in CON must be in the order
'       of the param entries here.
overlay_table
{00}daa_ovl_            long    0-0                        '\\ parameters for overlays (preset by spin code)
{01}ldi_ovl_            long    0-0                        '||   bits 31..16 = hub END+3 address (must be even length)
{02}cpi_ovl_            long    0-0                        '||   bits  8..0  = cog END address
{03}ini_ovl_            long    0-0                        '||
{04}outi_ovl_           long    0-0                        '||
{05}ldd_ovl_            long    0-0                        '||
{06}cpd_ovl_            long    0-0                        '||
{07}ind_ovl_            long    0-0                        '||
{08}outd_ovl_           long    0-0                        '||
{09}ldir_ovl_           long    0-0                        '||
{0A}cpir_ovl_           long    0-0                        '||
{0B}inir_ovl_           long    0-0                        '||
{0C}otir_ovl_           long    0-0                        '||
{0D}lddr_ovl_           long    0-0                        '||
{0E}cpdr_ovl_           long    0-0                        '||
{0F}indr_ovl_           long    0-0                        '||
{10}otdr_ovl_           long    0-0                        '||
{11}out_ovl_            long    0-0                        '||
{12}in_ovl_             long    0-0                        '||
{13}alu_neg_ovl_        long    0-0                        '||
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_00
{00}                    long    fetch                                                                     'NOP
{01}                    long    get_immediate_word + (put_bc              << 9)                           'LXI  B  *
{02}                    long    get_a              + (put_memory_bc       << 9)                           'STAX B  *
{03}                    long    get_bc             + (increment_word      << 9) + (put_bc          << 18) 'INX  B  *
{04}                    long    get_b              + (increment_byte      << 9) + (put_b           << 18) 'INR  B  *
{05}                    long    get_b              + (decrement_byte      << 9) + (put_b           << 18) 'DCR  B  *
{06}                    long    get_immediate_byte + (put_b               << 9)                           'MVI  B  *
{07}                    long    get_a              + (alu_rlc             << 9) + (put_a           << 18) 'RLC     *
{08}                    long    not_implemented                                                           'EX      AF,AF'
{09}                    long    get_bc             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  B  *
{0A}                    long    get_memory_bc      + (put_a               << 9)                           'LDAX B  *
{0B}                    long    get_bc             + (decrement_word      << 9) + (put_bc          << 18) 'DCX  B  *
{0C}                    long    get_c              + (increment_byte      << 9) + (put_c           << 18) 'INR  C  *
{0D}                    long    get_c              + (decrement_byte      << 9) + (put_c           << 18) 'DCR  C  *
{0E}                    long    get_immediate_byte + (put_c               << 9)                           'MVI  C  *
{0F}                    long    get_a              + (alu_rrc             << 9) + (put_a           << 18) 'RRC     *

{10}                    long    dec_jmp_nz                                                                'DJNZ disp     
{11}                    long    get_immediate_word + (put_de              << 9)                           'LXI  D  *
{12}                    long    get_a              + (put_memory_de       << 9)                           'STAX D  *
{13}                    long    get_de             + (increment_word      << 9) + (put_de          << 18) 'INX  D  *
{14}                    long    get_d              + (increment_byte      << 9) + (put_d           << 18) 'INR  D  *
{15}                    long    get_d              + (decrement_byte      << 9) + (put_d           << 18) 'DCR  D  *
{16}                    long    get_immediate_byte + (put_d               << 9)                           'MVI  D  *
{17}                    long    get_a              + (alu_ral             << 9) + (put_a           << 18) 'RAL     *
{18}                    long    jump_rel_unconditional                                                    'JR   disp
{19}                    long    get_de             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  D  *
{1A}                    long    get_memory_de      + (put_a               << 9)                           'LDAX D  *
{1B}                    long    get_de             + (decrement_word      << 9) + (put_de          << 18) 'DCX  D  *
{1C}                    long    get_e              + (increment_byte      << 9) + (put_e           << 18) 'INR  E  *
{1D}                    long    get_e              + (decrement_byte      << 9) + (put_e           << 18) 'DCR  E  *
{1E}                    long    get_immediate_byte + (put_e               << 9)                           'MVI  E  *
{1F}                    long    get_a              + (alu_rar             << 9) + (put_a           << 18) 'RAR     *

{20}                    long    test_flag_false    + (jump_rel_conditional<< 9) + (zero_bit        << 18) 'JR   NZ,disp
{21}                    long    get_immediate_word + (put_hl              << 9)                           'LXI  H  *
{22}                    long    get_hl             + (store_word_direct   << 9)                           'SHLD    *
{23}                    long    get_hl             + (increment_word      << 9) + (put_hl          << 18) 'INX  H  *
{24}                    long    get_h              + (increment_byte      << 9) + (put_h           << 18) 'INR  H  *
{25}                    long    get_h              + (decrement_byte      << 9) + (put_h           << 18) 'DCR  H  *
{26}                    long    get_immediate_byte + (put_h               << 9)                           'MVI  H  *
{27}                    long    vector_overlay     + (daa_ovl_no          << 9)                           'DAA
{28}                    long    test_flag_true     + (jump_rel_conditional<< 9) + (zero_bit        << 18) 'JR   Z,disp
{29}                    long    get_hl             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  H  *
{2A}                    long    load_word_direct   + (put_hl              << 9)                           'LHLD    *
{2B}                    long    get_hl             + (decrement_word      << 9) + (put_hl          << 18) 'DCX  H  *
{2C}                    long    get_l              + (increment_byte      << 9) + (put_l           << 18) 'INR  L  *
{2D}                    long    get_l              + (decrement_byte      << 9) + (put_l           << 18) 'DCR  L  *
{2E}                    long    get_immediate_byte + (put_l               << 9)                           'MVI  L  *
{2F}                    long    get_a              + (complement_byte     << 9) + (put_a           << 18) 'CMA     *

{30}                    long    test_flag_false    + (jump_rel_conditional<< 9) + (carry_bit       << 18) 'JR   NC,disp
{31}                    long    get_immediate_word + (put_sp              << 9)                           'LXI  SP *
{32}                    long    get_immediate_word + (store_a_direct      << 9)                           'STA     *
{33}                    long    get_sp             + (increment_word      << 9) + (put_sp          << 18) 'INX  SP *
{34}                    long    get_memory_hl      + (increment_byte      << 9) + (put_memory_hl   << 18) 'INR  M  *
{35}                    long    get_memory_hl      + (decrement_byte      << 9) + (put_memory_hl   << 18) 'DCR  M  *
{36}                    long    get_immediate_byte + (put_memory_hl       << 9)                           'MVI  M  *
{37}                    long    set_carry                                                                 'STC     *
{38}                    long    test_flag_true     + (jump_rel_conditional<< 9) + (carry_bit       << 18) 'JR   C,disp
{39}                    long    get_sp             + (double_add_hl       << 9) + (put_hl          << 18) 'DAD  SP *
{3A}                    long    get_immediate_word + (load_a_direct       << 9)                           'LDA     *
{3B}                    long    get_sp             + (decrement_word      << 9) + (put_sp          << 18) 'DCX  SP *
{3C}                    long    get_a              + (increment_byte      << 9) + (put_a           << 18) 'INR  A  *
{3D}                    long    get_a              + (decrement_byte      << 9) + (put_a           << 18) 'DCR  A  *
{3E}                    long    get_immediate_byte + (put_a               << 9)                           'MVI  A  *
{3F}                    long    complement_carry                                                          'CMC     *


{40}                    long    fetch                                                                     'MOV B,B
{41}                    long    get_c              + (put_b               << 9)                           'MOV B,C
{42}                    long    get_d              + (put_b               << 9)                           'MOV B,D
{43}                    long    get_e              + (put_b               << 9)                           'MOV B,E
{44}                    long    get_h              + (put_b               << 9)                           'MOV B,H
{45}                    long    get_l              + (put_b               << 9)                           'MOV B,L
{46}                    long    get_memory_hl      + (put_b               << 9)                           'MOV B,M
{47}                    long    get_a              + (put_b               << 9)                           'MOV B,A
{48}                    long    get_b              + (put_c               << 9)                           'MOV C,B
{49}                    long    fetch                                                                     'MOV C,C
{4A}                    long    get_d              + (put_c               << 9)                           'MOV C,D
{4B}                    long    get_e              + (put_c               << 9)                           'MOV C,E
{4C}                    long    get_h              + (put_c               << 9)                           'MOV C,H
{4D}                    long    get_l              + (put_c               << 9)                           'MOV C,L
{4E}                    long    get_memory_hl      + (put_c               << 9)                           'MOV C,M
{4F}                    long    get_a              + (put_c               << 9)                           'MOV C,A

{50}                    long    get_b              + (put_d               << 9)                           'MOV D,B
{51}                    long    get_c              + (put_d               << 9)                           'MOV D,C
{52}                    long    fetch                                                                     'MOV D,D
{53}                    long    get_e              + (put_d               << 9)                           'MOV D,E
{54}                    long    get_h              + (put_d               << 9)                           'MOV D,H
{55}                    long    get_l              + (put_d               << 9)                           'MOV D,L
{56}                    long    get_memory_hl      + (put_d               << 9)                           'MOV D,M
{57}                    long    get_a              + (put_d               << 9)                           'MOV D,A
{58}                    long    get_b              + (put_e               << 9)                           'MOV E,B
{59}                    long    get_c              + (put_e               << 9)                           'MOV E,C
{5A}                    long    get_d              + (put_e               << 9)                           'MOV E,D
{5B}                    long    fetch                                                                     'MOV E,E
{5C}                    long    get_h              + (put_e               << 9)                           'MOV E,H
{5D}                    long    get_l              + (put_e               << 9)                           'MOV E,L
{5E}                    long    get_memory_hl      + (put_e               << 9)                           'MOV E,M
{5F}                    long    get_a              + (put_e               << 9)                           'MOV E,A

{60}                    long    get_b              + (put_h               << 9)                           'MOV H,B
{61}                    long    get_c              + (put_h               << 9)                           'MOV H,C
{62}                    long    get_d              + (put_h               << 9)                           'MOV H,D
{63}                    long    get_e              + (put_h               << 9)                           'MOV H,E
{64}                    long    fetch                                                                     'MOV H,H
{65}                    long    get_l              + (put_h               << 9)                           'MOV H,L
{66}                    long    get_memory_hl      + (put_h               << 9)                           'MOV H,M
{67}                    long    get_a              + (put_h               << 9)                           'MOV H,A
{68}                    long    get_b              + (put_l               << 9)                           'MOV L,B
{69}                    long    get_c              + (put_l               << 9)                           'MOV L,C
{6A}                    long    get_d              + (put_l               << 9)                           'MOV L,D
{6B}                    long    get_e              + (put_l               << 9)                           'MOV L,E
{6C}                    long    get_h              + (put_l               << 9)                           'MOV L,H
{6D}                    long    fetch                                                                     'MOV L,L
{6E}                    long    get_memory_hl      + (put_l               << 9)                           'MOV L,M
{6F}                    long    get_a              + (put_l               << 9)                           'MOV L,A

{70}                    long    get_b              + (put_memory_hl       << 9)                           'MOV M,B
{71}                    long    get_c              + (put_memory_hl       << 9)                           'MOV M,C
{72}                    long    get_d              + (put_memory_hl       << 9)                           'MOV M,D
{73}                    long    get_e              + (put_memory_hl       << 9)                           'MOV M,E
{74}                    long    get_h              + (put_memory_hl       << 9)                           'MOV M,H
{75}                    long    get_l              + (put_memory_hl       << 9)                           'MOV M,L
{76}                    long    halt                                                                      'HALT
{77}                    long    get_a              + (put_memory_hl       << 9)                           'MOV M,A
{78}                    long    get_b              + (put_a               << 9)                           'MOV A,B
{79}                    long    get_c              + (put_a               << 9)                           'MOV A,C
{7A}                    long    get_d              + (put_a               << 9)                           'MOV A,D
{7B}                    long    get_e              + (put_a               << 9)                           'MOV A,E
{7C}                    long    get_h              + (put_a               << 9)                           'MOV A,H
{7D}                    long    get_l              + (put_a               << 9)                           'MOV A,L
{7E}                    long    get_memory_hl      + (put_a               << 9)                           'MOV A,M
{7F}                    long    fetch                                                                     'MOV A,A


{80}                    long    get_b              + (alu_add             << 9) + (put_a           << 18) 'ADD B *
{81}                    long    get_c              + (alu_add             << 9) + (put_a           << 18) 'ADD C *
{82}                    long    get_d              + (alu_add             << 9) + (put_a           << 18) 'ADD D *
{83}                    long    get_e              + (alu_add             << 9) + (put_a           << 18) 'ADD E *
{84}                    long    get_h              + (alu_add             << 9) + (put_a           << 18) 'ADD H *
{85}                    long    get_l              + (alu_add             << 9) + (put_a           << 18) 'ADD L *
{86}                    long    get_memory_hl      + (alu_add             << 9) + (put_a           << 18) 'ADD M *
{87}                    long    get_a              + (alu_add             << 9) + (put_a           << 18) 'ADD A *
{88}                    long    get_b              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC B *
{89}                    long    get_c              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC C *
{8A}                    long    get_d              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC D *
{8B}                    long    get_e              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC E *
{8C}                    long    get_h              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC H *
{8D}                    long    get_l              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC L *
{8E}                    long    get_memory_hl      + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC M *
{8F}                    long    get_a              + (alu_add_with_carry  << 9) + (put_a           << 18) 'ADC A *

{90}                    long    get_b              + (alu_sub             << 9) + (put_a           << 18) 'SUB B * 
{91}                    long    get_c              + (alu_sub             << 9) + (put_a           << 18) 'SUB C *
{92}                    long    get_d              + (alu_sub             << 9) + (put_a           << 18) 'SUB D *
{93}                    long    get_e              + (alu_sub             << 9) + (put_a           << 18) 'SUB E *
{94}                    long    get_h              + (alu_sub             << 9) + (put_a           << 18) 'SUB H *
{95}                    long    get_l              + (alu_sub             << 9) + (put_a           << 18) 'SUB L *
{96}                    long    get_memory_hl      + (alu_sub             << 9) + (put_a           << 18) 'SUB M *
{97}                    long    get_a              + (alu_sub             << 9) + (put_a           << 18) 'SUB A *
{98}                    long    get_b              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB B *
{99}                    long    get_c              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB C *
{9A}                    long    get_d              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB D *
{9B}                    long    get_e              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB E *
{9C}                    long    get_h              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB H *
{9D}                    long    get_l              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB L *
{9E}                    long    get_memory_hl      + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB M *
{9F}                    long    get_a              + (alu_sub_with_borrow << 9) + (put_a           << 18) 'SBB A *

{A0}                    long    get_b              + (alu_and             << 9) + (put_a           << 18) 'AND B * 
{A1}                    long    get_c              + (alu_and             << 9) + (put_a           << 18) 'AND C *
{A2}                    long    get_d              + (alu_and             << 9) + (put_a           << 18) 'AND D *
{A3}                    long    get_e              + (alu_and             << 9) + (put_a           << 18) 'AND E *
{A4}                    long    get_h              + (alu_and             << 9) + (put_a           << 18) 'AND H *
{A5}                    long    get_l              + (alu_and             << 9) + (put_a           << 18) 'AND L *
{A6}                    long    get_memory_hl      + (alu_and             << 9) + (put_a           << 18) 'AND M *
{A7}                    long    get_a              + (alu_and             << 9) + (put_a           << 18) 'AND A *
{A8}                    long    get_b              + (alu_xor             << 9) + (put_a           << 18) 'XRA B *
{A9}                    long    get_c              + (alu_xor             << 9) + (put_a           << 18) 'XRA C *
{AA}                    long    get_d              + (alu_xor             << 9) + (put_a           << 18) 'XRA D *
{AB}                    long    get_e              + (alu_xor             << 9) + (put_a           << 18) 'XRA E *
{AC}                    long    get_h              + (alu_xor             << 9) + (put_a           << 18) 'XRA H *
{AD}                    long    get_l              + (alu_xor             << 9) + (put_a           << 18) 'XRA L *
{AE}                    long    get_memory_hl      + (alu_xor             << 9) + (put_a           << 18) 'XRA M *
{AF}                    long    get_a              + (alu_xor             << 9) + (put_a           << 18) 'XRA A *

{B0}                    long    get_b              + (alu_or              << 9) + (put_a           << 18) 'ORA B *
{B1}                    long    get_c              + (alu_or              << 9) + (put_a           << 18) 'ORA C *
{B2}                    long    get_d              + (alu_or              << 9) + (put_a           << 18) 'ORA D *
{B3}                    long    get_e              + (alu_or              << 9) + (put_a           << 18) 'ORA E *
{B4}                    long    get_h              + (alu_or              << 9) + (put_a           << 18) 'ORA H *
{B5}                    long    get_l              + (alu_or              << 9) + (put_a           << 18) 'ORA L *
{B6}                    long    get_memory_hl      + (alu_or              << 9) + (put_a           << 18) 'ORA M *
{B7}                    long    get_a              + (alu_or              << 9) + (put_a           << 18) 'ORA A *
{B8}                    long    get_b              + (alu_sub             << 9) + (fetch           << 18) 'CMP B * FIXME cmp has no result !!!
{B9}                    long    get_c              + (alu_sub             << 9) + (fetch           << 18) 'CMP C *
{BA}                    long    get_d              + (alu_sub             << 9) + (fetch           << 18) 'CMP D *
{BB}                    long    get_e              + (alu_sub             << 9) + (fetch           << 18) 'CMP E *
{BC}                    long    get_h              + (alu_sub             << 9) + (fetch           << 18) 'CMP H *
{BD}                    long    get_l              + (alu_sub             << 9) + (fetch           << 18) 'CMP L *
{BE}                    long    get_memory_hl      + (alu_sub             << 9) + (fetch           << 18) 'CMP M *
{BF}                    long    get_a              + (alu_sub             << 9) + (fetch           << 18) 'CMP A *

{C0}                    long    test_flag_false    + (return_conditional  << 9) + (zero_bit        << 18) '*RNZ 
{C1}                    long    pop_               + (put_bc              << 9)                           '* POP B
{C2}                    long    test_flag_false    + (jump_conditional    << 9) + (zero_bit        << 18) '* JNZ 
{C3}                    long    jump_unconditional                                                        '* JMP 
{C4}                    long    test_flag_false    + (call_conditional    << 9) + (zero_bit        << 18) '*CNZ 
{C5}                    long    get_bc             + (push_               << 9)                           '*PUSH B
{C6}                    long    get_immediate_byte + (alu_add             << 9) + (put_a           << 18) '*ADI 
{C7}                    long    restart            + ((0 << 3)            << 9)                           '*RST 0
{C8}                    long    test_flag_true     + (return_conditional  << 9) + (zero_bit        << 18) '* RZ 
{C9}                    long    return_unconditional                                                      '*RET 
{CA}                    long    test_flag_true     + (jump_conditional    << 9) + (zero_bit        << 18) '*JZ 
{CB}                    long    z80_CB                                                                    '*Z80, CB Prefix 
{CC}                    long    test_flag_true     + (call_conditional    << 9) + (zero_bit        << 18) '*CZ 
{CD}                    long    call_unconditional                                                        '*CALL 
{CE}                    long    get_immediate_byte + (alu_add_with_carry  << 9) + (put_a           << 18) '*ACI 
{CF}                    long    restart            + ((1 << 3)            << 9)                           '*RST 1

{D0}                    long    test_flag_false    + (return_conditional  << 9) + (carry_bit       << 18) '*RNC 
{D1}                    long    pop_               + (put_de              << 9)                           '*POP D
{D2}                    long    test_flag_false    + (jump_conditional    << 9) + (carry_bit       << 18) '*JNC 
{D3}                    long    vector_overlay     + (out_ovl_no          << 9)                           '*OUT
{D4}                    long    test_flag_false    + (call_conditional    << 9) + (carry_bit       << 18) '*CNC 
{D5}                    long    get_de             + (push_               << 9)                           '*PUSH D
{D6}                    long    get_immediate_byte + (alu_sub             << 9) + (put_a           << 18) '*SUI
{D7}                    long    restart            + ((2 << 3)            << 9)                           '*RST 2
{D8}                    long    test_flag_true     + (return_conditional  << 9) + (carry_bit       << 18) '*RC 
{D9}                    long    z80_exx                                                                   '*EXX, D9, BC/DE/HL <-> BC'/DE'/HL' 
{DA}                    long    test_flag_true     + (jump_conditional    << 9) + (carry_bit       << 18) '*JC 
{DB}                    long    vector_overlay     + (in_ovl_no           << 9)                           'IN
{DC}                    long    test_flag_true     + (call_conditional    << 9) + (carry_bit       << 18) '*CC 
{DD}                    long    z80_DD                                                                    '*Z80 DD Prefix 
{DE}                    long    get_immediate_byte + (alu_sub_with_borrow << 9) + (put_a           << 18) '*SBI 
{DF}                    long    restart            + ((3 << 3)            << 9)                           '*RST 3

{E0}                    long    test_flag_false    + (return_conditional  << 9) + (parity_bit      << 18) '*RPO 
{E1}                    long    pop_               + (put_hl              << 9)                           '*POP H
{E2}                    long    test_flag_false    + (jump_conditional    << 9) + (parity_bit      << 18) '*JPO 
{E3}                    long    get_tos            + (exchange            << 9) + (put_tos         << 18) '*XTHL,    H <-> (SP+1); L <-> (SP) 
{E4}                    long    test_flag_false    + (call_conditional    << 9) + (parity_bit      << 18) '*CPO 
{E5}                    long    get_hl             + (push_               << 9)                           '*PUSH H
{E6}                    long    get_immediate_byte + (alu_and             << 9) + (put_a           << 18) '*ANI 
{E7}                    long    restart            + ((4 << 3)            << 9)                           '*RST 4
{E8}                    long    test_flag_true     + (return_conditional  << 9) + (parity_bit      << 18) '*RPE 
{E9}                    long    get_hl             + (put_pc              << 9)                           '*PCHL,    PC <- HL
{EA}                    long    test_flag_true     + (jump_conditional    << 9) + (parity_bit      << 18) '*JPE 
{EB}                    long    get_de             + (exchange            << 9) + (put_de          << 18) '*XCHG,    HL <-> DE  
{EC}                    long    test_flag_true     + (call_conditional    << 9) + (parity_bit      << 18) '*CPE 
{ED}                    long    z80_ED                                                                    '*Z80 ED Prefix 
{EE}                    long    get_immediate_byte + (alu_xor             << 9) + (put_a           << 18) '*XRI 
{EF}                    long    restart            + ((5 << 3)            << 9)                           '*RST 5

{F0}                    long    test_flag_false    + (return_conditional  << 9) + (sign_bit        << 18) '*RP 
{F1}                    long    pop_               + (put_af              << 9)                           '*POP AF
{F2}                    long    test_flag_false    + (jump_conditional    << 9) + (sign_bit        << 18) '*JP 
{F3}                    long    not_implemented                                                           '*DI,  Disable interrupts 
{F4}                    long    test_flag_false    + (call_conditional    << 9) + (sign_bit        << 18) '*CP 
{F5}                    long    get_af             + (push_               << 9)                           '*PUSH AF
{F6}                    long    get_immediate_byte + (alu_or              << 9) + (put_a           << 18) '*ORI 
{F7}                    long    restart            + ((6 << 3)            << 9)                           '*RST 6
{F8}                    long    test_flag_true     + (return_conditional  << 9) + (sign_bit        << 18) '*RM 
{F9}                    long    get_hl             + (put_sp              << 9)                           '*SPHL,    SP <- HL 
{FA}                    long    test_flag_true     + (jump_conditional    << 9) + (sign_bit        << 18) '*JM 
{FB}                    long    not_implemented                                                           '*EI, Enable interrupts 
{FC}                    long    test_flag_true     + (call_conditional    << 9) + (sign_bit        << 18) '*CM 
{FD}                    long    z80_FD                                                                    '*Z80 FD Prefix 
{FE}                    long    get_immediate_byte + (alu_sub             << 9) + (fetch           << 18) '*CPI
{FF}                    long    restart            + ((7 << 3)            << 9)                           '*RST 7
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_cb
{00}                    long    get_b              + (alu_rlc             << 9) + (put_b           << 18) 'RLC     B               CB00            ---
{01}                    long    get_c              + (alu_rlc             << 9) + (put_c           << 18) 'RLC     C               CB01            ---
{02}                    long    get_d              + (alu_rlc             << 9) + (put_d           << 18) 'RLC     D               CB02            ---
{03}                    long    get_e              + (alu_rlc             << 9) + (put_e           << 18) 'RLC     E               CB03            ---
{04}                    long    get_h              + (alu_rlc             << 9) + (put_h           << 18) 'RLC     H               CB04            ---
{05}                    long    get_l              + (alu_rlc             << 9) + (put_l           << 18) 'RLC     L               CB05            ---
{06}                    long    get_memory_hl      + (alu_rlc             << 9) + (put_memory_hl   << 18) 'RLC     (HL)            CB06            ---
{07}                    long    get_a              + (alu_rlc             << 9) + (put_a           << 18) 'RLC     A               CB07            ---
{08}                    long    get_b              + (alu_rrc             << 9) + (put_b           << 18) 'RRC     B               CB08            ---
{09}                    long    get_c              + (alu_rrc             << 9) + (put_c           << 18) 'RRC     C               CB09            ---
{0A}                    long    get_d              + (alu_rrc             << 9) + (put_d           << 18) 'RRC     D               CB0A            ---
{0B}                    long    get_e              + (alu_rrc             << 9) + (put_e           << 18) 'RRC     E               CB0B            ---
{0C}                    long    get_h              + (alu_rrc             << 9) + (put_h           << 18) 'RRC     H               CB0C            ---
{0D}                    long    get_l              + (alu_rrc             << 9) + (put_l           << 18) 'RRC     L               CB0D            ---
{0E}                    long    get_memory_hl      + (alu_rrc             << 9) + (put_memory_hl   << 18) 'RRC     (HL)            CB0E            ---
{0F}                    long    get_a              + (alu_rrc             << 9) + (put_a           << 18) 'RRC     A               CB0F            ---

{10}                    long    get_b              + (alu_ral             << 9) + (put_b           << 18) 'RL      B               CB10            ---
{11}                    long    get_c              + (alu_ral             << 9) + (put_c           << 18) 'RL      C               CB11            ---
{12}                    long    get_d              + (alu_ral             << 9) + (put_d           << 18) 'RL      D               CB12            ---
{13}                    long    get_e              + (alu_ral             << 9) + (put_e           << 18) 'RL      E               CB13            ---
{14}                    long    get_h              + (alu_ral             << 9) + (put_h           << 18) 'RL      H               CB14            ---
{15}                    long    get_l              + (alu_ral             << 9) + (put_l           << 18) 'RL      L               CB15            ---
{16}                    long    get_memory_hl      + (alu_ral             << 9) + (put_memory_hl   << 18) 'RL      (HL)            CB16            ---
{17}                    long    get_a              + (alu_ral             << 9) + (put_a           << 18) 'RL      A               CB17            ---
{18}                    long    get_b              + (alu_rar             << 9) + (put_b           << 18) 'RR      B               CB18            ---
{19}                    long    get_c              + (alu_rar             << 9) + (put_c           << 18) 'RR      C               CB19            ---
{1A}                    long    get_d              + (alu_rar             << 9) + (put_d           << 18) 'RR      D               CB1A            ---
{1B}                    long    get_e              + (alu_rar             << 9) + (put_e           << 18) 'RR      E               CB1B            ---
{1C}                    long    get_h              + (alu_rar             << 9) + (put_h           << 18) 'RR      H               CB1C            ---
{1D}                    long    get_l              + (alu_rar             << 9) + (put_l           << 18) 'RR      L               CB1D            ---
{1E}                    long    get_memory_hl      + (alu_rar             << 9) + (put_memory_hl   << 18) 'RR      (HL)            CB1E            ---
{1F}                    long    get_a              + (alu_rar             << 9) + (put_a           << 18) 'RR      A               CB1F            ---

{20}                    long    get_b              + (alu_sla             << 9) + (put_b           << 18) 'SLA     B               CB20            ---
{21}                    long    get_c              + (alu_sla             << 9) + (put_c           << 18) 'SLA     C               CB21            ---
{22}                    long    get_d              + (alu_sla             << 9) + (put_d           << 18) 'SLA     D               CB22            ---
{23}                    long    get_e              + (alu_sla             << 9) + (put_e           << 18) 'SLA     E               CB23            ---
{24}                    long    get_h              + (alu_sla             << 9) + (put_h           << 18) 'SLA     H               CB24            ---
{25}                    long    get_l              + (alu_sla             << 9) + (put_l           << 18) 'SLA     L               CB25            ---
{26}                    long    get_memory_hl      + (alu_sla             << 9) + (put_memory_hl   << 18) 'SLA     (HL)            CB26            ---
{27}                    long    get_a              + (alu_sla             << 9) + (put_a           << 18) 'SLA     A               CB27            ---
{28}                    long    get_b              + (alu_sra             << 9) + (put_b           << 18) 'SRA     B               CB28            ---
{29}                    long    get_c              + (alu_sra             << 9) + (put_c           << 18) 'SRA     C               CB29            ---
{2A}                    long    get_d              + (alu_sra             << 9) + (put_d           << 18) 'SRA     D               CB2A            ---
{2B}                    long    get_e              + (alu_sra             << 9) + (put_e           << 18) 'SRA     E               CB2B            ---
{2C}                    long    get_h              + (alu_sra             << 9) + (put_h           << 18) 'SRA     H               CB2C            ---
{2D}                    long    get_l              + (alu_sra             << 9) + (put_l           << 18) 'SRA     L               CB2D            ---
{2E}                    long    get_memory_hl      + (alu_sra             << 9) + (put_memory_hl   << 18) 'SRA     (HL)            CB2E            ---
{2F}                    long    get_a              + (alu_sra             << 9) + (put_a           << 18) 'SRA     A               CB2F            ---

{30}                    long    undocumented
{31}                    long    undocumented
{32}                    long    undocumented
{33}                    long    undocumented
{34}                    long    undocumented
{35}                    long    undocumented
{36}                    long    undocumented
{37}                    long    undocumented
{38}                    long    get_b              + (alu_srl             << 9) + (put_b           << 18) 'SRL     B               CB38            ---
{39}                    long    get_c              + (alu_srl             << 9) + (put_c           << 18) 'SRL     C               CB39            ---
{3A}                    long    get_d              + (alu_srl             << 9) + (put_d           << 18) 'SRL     D               CB3A            ---
{3B}                    long    get_e              + (alu_srl             << 9) + (put_e           << 18) 'SRL     E               CB3B            ---
{3C}                    long    get_h              + (alu_srl             << 9) + (put_h           << 18) 'SRL     H               CB3C            ---
{3D}                    long    get_l              + (alu_srl             << 9) + (put_l           << 18) 'SRL     L               CB3D            ---
{3E}                    long    get_memory_hl      + (alu_srl             << 9) + (put_memory_hl   << 18) 'SRL     (HL)            CB3E            ---
{3F}                    long    get_a              + (alu_srl             << 9) + (put_a           << 18) 'SRL     A               CB3F            ---

{40}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (0 << 27) 'BIT     0,B             CB40            Z flag <- NOT 0b
{41}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (0 << 27) 'BIT     0,C             CB41            Z flag <- NOT 0b
{42}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (0 << 27) 'BIT     0,D             CB42            Z flag <- NOT 0b
{43}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (0 << 27) 'BIT     0,E             CB43            Z flag <- NOT 0b
{44}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (0 << 27) 'BIT     0,H             CB44            Z flag <- NOT 0b
{45}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (0 << 27) 'BIT     0,L             CB45            Z flag <- NOT 0b
{46}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (0 << 27) 'BIT     0,(HL)          CB46            Z flag <- NOT 0b
{47}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (0 << 27) 'BIT     0,A             CB47            Z flag <- NOT 0b
{48}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (1 << 27) 'BIT     1,B             CB48            Z flag <- NOT 1b
{49}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (1 << 27) 'BIT     1,C             CB49            Z flag <- NOT 1b
{4A}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (1 << 27) 'BIT     1,D             CB4A            Z flag <- NOT 1b
{4B}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (1 << 27) 'BIT     1,E             CB4B            Z flag <- NOT 1b
{4C}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (1 << 27) 'BIT     1,H             CB4C            Z flag <- NOT 1b
{4D}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (1 << 27) 'BIT     1,L             CB4D            Z flag <- NOT 1b
{4E}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (1 << 27) 'BIT     1,(HL)          CB4E            Z flag <- NOT 1b
{4F}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (1 << 27) 'BIT     1,A             CB4F            Z flag <- NOT 1b

{50}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (2 << 27) 'BIT     2,B             CB50            Z flag <- NOT 2b
{51}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (2 << 27) 'BIT     2,C             CB51            Z flag <- NOT 2b
{52}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (2 << 27) 'BIT     2,D             CB52            Z flag <- NOT 2b
{53}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (2 << 27) 'BIT     2,E             CB53            Z flag <- NOT 2b
{54}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (2 << 27) 'BIT     2,H             CB54            Z flag <- NOT 2b
{55}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (2 << 27) 'BIT     2,L             CB55            Z flag <- NOT 2b
{56}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (2 << 27) 'BIT     2,(HL)          CB56            Z flag <- NOT 2b
{57}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (2 << 27) 'BIT     2,A             CB57            Z flag <- NOT 2b
{58}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (3 << 27) 'BIT     3,B             CB58            Z flag <- NOT 3b
{59}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (3 << 27) 'BIT     3,C             CB59            Z flag <- NOT 3b
{5A}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (3 << 27) 'BIT     3,D             CB5A            Z flag <- NOT 3b
{5B}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (3 << 27) 'BIT     3,E             CB5B            Z flag <- NOT 3b
{5C}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (3 << 27) 'BIT     3,H             CB5C            Z flag <- NOT 3b
{5D}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (3 << 27) 'BIT     3,L             CB5D            Z flag <- NOT 3b
{5E}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (3 << 27) 'BIT     3,(HL)          CB5E            Z flag <- NOT 3b
{5F}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (3 << 27) 'BIT     3,A             CB5F            Z flag <- NOT 3b

{60}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (4 << 27) 'BIT     4,B             CB60            Z flag <- NOT 4b
{61}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (4 << 27) 'BIT     4,C             CB61            Z flag <- NOT 4b
{62}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (4 << 27) 'BIT     4,D             CB62            Z flag <- NOT 4b
{63}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (4 << 27) 'BIT     4,E             CB63            Z flag <- NOT 4b
{64}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (4 << 27) 'BIT     4,H             CB64            Z flag <- NOT 4b
{65}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (4 << 27) 'BIT     4,L             CB65            Z flag <- NOT 4b
{66}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (4 << 27) 'BIT     4,(HL)          CB66            Z flag <- NOT 4b
{67}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (4 << 27) 'BIT     4,A             CB67            Z flag <- NOT 4b
{68}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (5 << 27) 'BIT     5,B             CB68            Z flag <- NOT 5b
{69}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (5 << 27) 'BIT     5,C             CB69            Z flag <- NOT 5b
{6A}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (5 << 27) 'BIT     5,D             CB6A            Z flag <- NOT 5b
{6B}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (5 << 27) 'BIT     5,E             CB6B            Z flag <- NOT 5b
{6C}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (5 << 27) 'BIT     5,H             CB6C            Z flag <- NOT 5b
{6D}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (5 << 27) 'BIT     5,L             CB6D            Z flag <- NOT 5b
{6E}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (5 << 27) 'BIT     5,(HL)          CB6E            Z flag <- NOT 5b
{6F}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (5 << 27) 'BIT     5,A             CB6F            Z flag <- NOT 5b

{70}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (6 << 27) 'BIT     6,B             CB70            Z flag <- NOT 6b
{71}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (6 << 27) 'BIT     6,C             CB71            Z flag <- NOT 6b
{72}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (6 << 27) 'BIT     6,D             CB72            Z flag <- NOT 6b
{73}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (6 << 27) 'BIT     6,E             CB73            Z flag <- NOT 6b
{74}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (6 << 27) 'BIT     6,H             CB74            Z flag <- NOT 6b
{75}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (6 << 27) 'BIT     6,L             CB75            Z flag <- NOT 6b
{76}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (6 << 27) 'BIT     6,(HL)          CB76            Z flag <- NOT 6b
{77}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (6 << 27) 'BIT     6,A             CB77            Z flag <- NOT 6b
{78}                    long    get_b              + (alu_bit             << 9) + (put_b           << 18) + (7 << 27) 'BIT     7,B             CB78            Z flag <- NOT 7b
{79}                    long    get_c              + (alu_bit             << 9) + (put_c           << 18) + (7 << 27) 'BIT     7,C             CB79            Z flag <- NOT 7b
{7A}                    long    get_d              + (alu_bit             << 9) + (put_d           << 18) + (7 << 27) 'BIT     7,D             CB7A            Z flag <- NOT 7b
{7B}                    long    get_e              + (alu_bit             << 9) + (put_e           << 18) + (7 << 27) 'BIT     7,E             CB7B            Z flag <- NOT 7b
{7C}                    long    get_h              + (alu_bit             << 9) + (put_h           << 18) + (7 << 27) 'BIT     7,H             CB7C            Z flag <- NOT 7b
{7D}                    long    get_l              + (alu_bit             << 9) + (put_l           << 18) + (7 << 27) 'BIT     7,L             CB7D            Z flag <- NOT 7b
{7E}                    long    get_m              + (alu_bit             << 9) + (put_m           << 18) + (7 << 27) 'BIT     7,(HL)          CB7E            Z flag <- NOT 7b
{7F}                    long    get_a              + (alu_bit             << 9) + (put_a           << 18) + (7 << 27) 'BIT     7,A             CB7F            Z flag <- NOT 7b

{80}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (0 << 27) 'RES     0,B             CB80            0b <- 0
{81}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (0 << 27) 'RES     0,C             CB81            0b <- 0
{82}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (0 << 27) 'RES     0,D             CB82            0b <- 0
{83}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (0 << 27) 'RES     0,E             CB83            0b <- 0
{84}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (0 << 27) 'RES     0,H             CB84            0b <- 0
{85}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (0 << 27) 'RES     0,L             CB85            0b <- 0
{86}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (0 << 27) 'RES     0,(HL)          CB86            0b <- 0
{87}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (0 << 27) 'RES     0,A             CB87            0b <- 0
{88}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (1 << 27) 'RES     1,B             CB88            1b <- 0
{89}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (1 << 27) 'RES     1,C             CB89            1b <- 0
{8A}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (1 << 27) 'RES     1,D             CB8A            1b <- 0
{8B}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (1 << 27) 'RES     1,E             CB8B            1b <- 0
{8C}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (1 << 27) 'RES     1,H             CB8C            1b <- 0
{8D}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (1 << 27) 'RES     1,L             CB8D            1b <- 0
{8E}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (1 << 27) 'RES     1,(HL)          CB8E            1b <- 0
{8F}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (1 << 27) 'RES     1,A             CB8F            1b <- 0

{90}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (2 << 27) 'RES     2,B             CB90            2b <- 0
{91}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (2 << 27) 'RES     2,C             CB91            2b <- 0
{92}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (2 << 27) 'RES     2,D             CB92            2b <- 0
{93}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (2 << 27) 'RES     2,E             CB93            2b <- 0
{94}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (2 << 27) 'RES     2,H             CB94            2b <- 0
{95}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (2 << 27) 'RES     2,L             CB95            2b <- 0
{96}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (2 << 27) 'RES     2,(HL)          CB96            2b <- 0
{97}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (2 << 27) 'RES     2,A             CB97            2b <- 0
{98}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (3 << 27) 'RES     3,B             CB98            3b <- 0
{99}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (3 << 27) 'RES     3,C             CB99            3b <- 0
{9A}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (3 << 27) 'RES     3,D             CB9A            3b <- 0
{9B}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (3 << 27) 'RES     3,E             CB9B            3b <- 0
{9C}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (3 << 27) 'RES     3,H             CB9C            3b <- 0
{9D}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (3 << 27) 'RES     3,L             CB9D            3b <- 0
{9E}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (3 << 27) 'RES     3,(HL)          CB9E            3b <- 0
{9F}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (3 << 27) 'RES     3,A             CB9F            3b <- 0

{A0}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (4 << 27) 'RES     4,B             CBA0            4b <- 0
{A1}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (4 << 27) 'RES     4,C             CBA1            4b <- 0
{A2}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (4 << 27) 'RES     4,D             CBA2            4b <- 0
{A3}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (4 << 27) 'RES     4,E             CBA3            4b <- 0
{A4}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (4 << 27) 'RES     4,H             CBA4            4b <- 0
{A5}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (4 << 27) 'RES     4,L             CBA5            4b <- 0
{A6}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (4 << 27) 'RES     4,(HL)          CBA6            4b <- 0
{A7}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (4 << 27) 'RES     4,A             CBA7            4b <- 0
{A8}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (5 << 27) 'RES     5,B             CBA8            5b <- 0
{A9}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (5 << 27) 'RES     5,C             CBA9            5b <- 0
{AA}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (5 << 27) 'RES     5,D             CBAA            5b <- 0
{AB}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (5 << 27) 'RES     5,E             CBAB            5b <- 0
{AC}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (5 << 27) 'RES     5,H             CBAC            5b <- 0
{AD}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (5 << 27) 'RES     5,L             CBAD            5b <- 0
{AE}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (5 << 27) 'RES     5,(HL)          CBAE            5b <- 0
{AF}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (5 << 27) 'RES     5,A             CBAF            5b <- 0

{B0}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (6 << 27) 'RES     6,B             CBB0            6b <- 0
{B1}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (6 << 27) 'RES     6,C             CBB1            6b <- 0
{B2}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (6 << 27) 'RES     6,D             CBB2            6b <- 0
{B3}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (6 << 27) 'RES     6,E             CBB3            6b <- 0
{B4}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (6 << 27) 'RES     6,H             CBB4            6b <- 0
{B5}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (6 << 27) 'RES     6,L             CBB5            6b <- 0
{B6}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (6 << 27) 'RES     6,(HL)          CBB6            6b <- 0
{B7}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (6 << 27) 'RES     6,A             CBB7            6b <- 0
{B8}                    long    get_b              + (alu_res             << 9) + (put_b           << 18) + (7 << 27) 'RES     7,B             CBB8            7b <- 0
{B9}                    long    get_c              + (alu_res             << 9) + (put_c           << 18) + (7 << 27) 'RES     7,C             CBB9            7b <- 0
{BA}                    long    get_d              + (alu_res             << 9) + (put_d           << 18) + (7 << 27) 'RES     7,D             CBBA            7b <- 0
{BB}                    long    get_e              + (alu_res             << 9) + (put_e           << 18) + (7 << 27) 'RES     7,E             CBBB            7b <- 0
{BC}                    long    get_h              + (alu_res             << 9) + (put_h           << 18) + (7 << 27) 'RES     7,H             CBBC            7b <- 0
{BD}                    long    get_l              + (alu_res             << 9) + (put_l           << 18) + (7 << 27) 'RES     7,L             CBBD            7b <- 0
{BE}                    long    get_m              + (alu_res             << 9) + (put_m           << 18) + (7 << 27) 'RES     7,(HL)          CBBE            7b <- 0
{BF}                    long    get_a              + (alu_res             << 9) + (put_a           << 18) + (7 << 27) 'RES     7,A             CBBF            7b <- 0

{C0}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (0 << 27) 'SET     0,B             CBC0            0b <- 1
{C1}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (0 << 27) 'SET     0,C             CBC1            0b <- 1
{C2}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (0 << 27) 'SET     0,D             CBC2            0b <- 1
{C3}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (0 << 27) 'SET     0,E             CBC3            0b <- 1
{C4}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (0 << 27) 'SET     0,H             CBC4            0b <- 1
{C5}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (0 << 27) 'SET     0,L             CBC5            0b <- 1
{C6}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (0 << 27) 'SET     0,(HL)          CBC6            0b <- 1
{C7}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (0 << 27) 'SET     0,A             CBC7            0b <- 1
{C8}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (1 << 27) 'SET     1,B             CBC8            1b <- 1
{C9}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (1 << 27) 'SET     1,C             CBC9            1b <- 1
{CA}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (1 << 27) 'SET     1,D             CBCA            1b <- 1
{CB}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (1 << 27) 'SET     1,E             CBCB            1b <- 1
{CC}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (1 << 27) 'SET     1,H             CBCC            1b <- 1
{CD}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (1 << 27) 'SET     1,L             CBCD            1b <- 1
{CE}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (1 << 27) 'SET     1,(HL)          CBCE            1b <- 1
{CF}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (1 << 27) 'SET     1,A             CBCF            1b <- 1

{D0}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (2 << 27) 'SET     2,B             CBD0            2b <- 1
{D1}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (2 << 27) 'SET     2,C             CBD1            2b <- 1
{D2}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (2 << 27) 'SET     2,D             CBD2            2b <- 1
{D3}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (2 << 27) 'SET     2,E             CBD3            2b <- 1
{D4}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (2 << 27) 'SET     2,H             CBD4            2b <- 1
{D5}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (2 << 27) 'SET     2,L             CBD5            2b <- 1
{D6}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (2 << 27) 'SET     2,(HL)          CBD6            2b <- 1
{D7}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (2 << 27) 'SET     2,A             CBD7            2b <- 1
{D8}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (3 << 27) 'SET     3,B             CBD8            3b <- 1
{D9}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (3 << 27) 'SET     3,C             CBD9            3b <- 1
{DA}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (3 << 27) 'SET     3,D             CBDA            3b <- 1
{DB}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (3 << 27) 'SET     3,E             CBDB            3b <- 1
{DC}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (3 << 27) 'SET     3,H             CBDC            3b <- 1
{DD}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (3 << 27) 'SET     3,L             CBDD            3b <- 1
{DE}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (3 << 27) 'SET     3,(HL)          CBDE            3b <- 1
{DF}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (3 << 27) 'SET     3,A             CBDF            3b <- 1

{E0}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (4 << 27) 'SET     4,B             CBE0            4b <- 1
{E1}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (4 << 27) 'SET     4,C             CBE1            4b <- 1
{E2}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (4 << 27) 'SET     4,D             CBE2            4b <- 1
{E3}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (4 << 27) 'SET     4,E             CBE3            4b <- 1
{E4}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (4 << 27) 'SET     4,H             CBE4            4b <- 1
{E5}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (4 << 27) 'SET     4,L             CBE5            4b <- 1
{E6}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (4 << 27) 'SET     4,(HL)          CBE6            4b <- 1
{E7}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (4 << 27) 'SET     4,A             CBE7            4b <- 1
{E8}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (5 << 27) 'SET     5,B             CBE8            5b <- 1
{E9}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (5 << 27) 'SET     5,C             CBE9            5b <- 1
{EA}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (5 << 27) 'SET     5,D             CBEA            5b <- 1
{EB}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (5 << 27) 'SET     5,E             CBEB            5b <- 1
{EC}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (5 << 27) 'SET     5,H             CBEC            5b <- 1
{ED}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (5 << 27) 'SET     5,L             CBED            5b <- 1
{EE}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (5 << 27) 'SET     5,(HL)          CBEE            5b <- 1
{EF}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (5 << 27) 'SET     5,A             CBEF            5b <- 1

{F0}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (6 << 27) 'SET     6,B             CBF0            6b <- 1
{F1}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (6 << 27) 'SET     6,C             CBF1            6b <- 1
{F2}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (6 << 27) 'SET     6,D             CBF2            6b <- 1
{F3}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (6 << 27) 'SET     6,E             CBF3            6b <- 1
{F4}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (6 << 27) 'SET     6,H             CBF4            6b <- 1
{F5}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (6 << 27) 'SET     6,L             CBF5            6b <- 1
{F6}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (6 << 27) 'SET     6,(HL)          CBF6            6b <- 1
{F7}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (6 << 27) 'SET     6,A             CBF7            6b <- 1
{F8}                    long    get_b              + (alu_set             << 9) + (put_b           << 18) + (7 << 27) 'SET     7,B             CBF8            7b <- 1
{F9}                    long    get_c              + (alu_set             << 9) + (put_c           << 18) + (7 << 27) 'SET     7,C             CBF9            7b <- 1
{FA}                    long    get_d              + (alu_set             << 9) + (put_d           << 18) + (7 << 27) 'SET     7,D             CBFA            7b <- 1
{FB}                    long    get_e              + (alu_set             << 9) + (put_e           << 18) + (7 << 27) 'SET     7,E             CBFB            7b <- 1
{FC}                    long    get_h              + (alu_set             << 9) + (put_h           << 18) + (7 << 27) 'SET     7,H             CBFC            7b <- 1
{FD}                    long    get_l              + (alu_set             << 9) + (put_l           << 18) + (7 << 27) 'SET     7,L             CBFD            7b <- 1
{FE}                    long    get_m              + (alu_set             << 9) + (put_m           << 18) + (7 << 27) 'SET     7,(HL)          CBFE            7b <- 1
{FF}                    long    get_a              + (alu_set             << 9) + (put_a           << 18) + (7 << 27) 'SET     7,A             CBFF            7b <- 1
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_dd
{00}                    long    undocumented
{01}                    long    undocumented
{02}                    long    undocumented
{03}                    long    undocumented
{04}                    long    undocumented
{05}                    long    undocumented
{06}                    long    undocumented
{07}                    long    undocumented
{08}                    long    undocumented                                
{09}                    long    get_bc             + (double_add_ixy      << 9) + (put_ixy        << 18)              '* ADD     IX,BC           DD09            IX <- IX + BC
{0A}                    long    undocumented
{0B}                    long    undocumented
{0C}                    long    undocumented
{0D}                    long    undocumented
{0E}                    long    undocumented
{0F}                    long    undocumented

{10}                    long    undocumented
{11}                    long    undocumented
{12}                    long    undocumented
{13}                    long    undocumented
{14}                    long    undocumented
{15}                    long    undocumented
{16}                    long    undocumented
{17}                    long    undocumented
{18}                    long    undocumented
{19}                    long    get_de             + (double_add_ixy      << 9) + (put_ixy        << 18)              '* ADD     IX,DE           DD19            IX <- IX + DE
{1A}                    long    undocumented
{1B}                    long    undocumented
{1C}                    long    undocumented
{1D}                    long    undocumented
{1E}                    long    undocumented
{1F}                    long    undocumented

{20}                    long    undocumented
{21}                    long    get_immediate_word + (put_ixy             << 9)                                       '* LD      IX,word         DD21word        IX <- word
{22}                    long    get_ixy            + (store_word_direct   << 9)                                       '* LD      (word),IX       DD22word        (word) <- IX
{23}                    long    get_ixy            + (increment_word      << 9) + (put_ixy        << 18)              '* INC     IX              DD23            IX <- IX + 1
{24}                    long    undocumented
{25}                    long    undocumented
{26}                    long    undocumented
{27}                    long    undocumented
{28}                    long    undocumented
{29}                    long    get_ixy            + (double_add_ixy      << 9) + (put_ixy        << 18)              '* ADD     IX,IX           DD29            IX <- IX + IX
{2A}                    long    load_word_direct   + (put_ixy             << 9)                                       '* LD      IX,(word)       DD2Aword        IX <- (word)
{2B}                    long    get_ixy            + (decrement_word      << 9) + (put_ixy        << 18)              '* DEC     IX              DD2B            IX <- IX - 1
{2C}                    long    undocumented
{2D}                    long    undocumented
{2E}                    long    undocumented
{2F}                    long    undocumented

{30}                    long    undocumented
{31}                    long    undocumented
{32}                    long    undocumented
{33}                    long    undocumented
{34}                    long    get_mem_xy_disp    + (increment_byte      << 9) + (put_memory_addr<< 18)              '** INC     (IX+disp)      DD34disp       (IX+disp) <- (IX+disp) + 1
{35}                    long    get_mem_xy_disp    + (decrement_byte      << 9) + (put_memory_addr<< 18)              '* DEC     (IX+disp)      DD35disp       (IX+disp) <- (IX+disp) - 1
{36}                    long    ld_ix_disp_byte                                                                       'LD      (IX+disp),byte DD76dispbyte   (IX+disp) <- byte
{37}                    long    undocumented
{38}                    long    undocumented
{39}                    long    get_sp             + (double_add_ixy      << 9) + (put_ixy        << 18)              '** ADD     IX,SP           DD39            IX <- IX + SP
{3A}                    long    undocumented
{3B}                    long    undocumented
{3C}                    long    undocumented
{3D}                    long    undocumented
{3E}                    long    undocumented
{3F}                    long    undocumented

{40}                    long    undocumented
{41}                    long    undocumented
{42}                    long    undocumented
{43}                    long    undocumented
{44}                    long    undocumented
{45}                    long    undocumented
{46}                    long    get_mem_xy_disp    + (put_b               << 9)                                       '** LD      B,(IX+disp)    DD46disp       B <- (IX+disp)
{47}                    long    undocumented
{48}                    long    undocumented
{49}                    long    undocumented
{4A}                    long    undocumented
{4B}                    long    undocumented
{4C}                    long    undocumented
{4D}                    long    undocumented
{4E}                    long    get_mem_xy_disp    + (put_c               << 9)                                       '* LD      C,(IX+disp)    DD4Edisp       C <- (IX+disp)
{4F}                    long    undocumented

{50}                    long    undocumented
{51}                    long    undocumented
{52}                    long    undocumented
{53}                    long    undocumented
{54}                    long    undocumented
{55}                    long    undocumented
{56}                    long    get_mem_xy_disp    + (put_d               << 9)                                       '* LD      D,(IX+disp)    DD56disp       D <- (IX+disp)
{57}                    long    undocumented
{58}                    long    undocumented
{59}                    long    undocumented
{5A}                    long    undocumented
{5B}                    long    undocumented
{5C}                    long    undocumented
{5D}                    long    undocumented
{5E}                    long    get_mem_xy_disp    + (put_e               << 9)                                       '* LD      E,(IX+disp)    DD5Edisp       E <- (IX+disp)
{5F}                    long    undocumented

{60}                    long    undocumented
{61}                    long    undocumented
{62}                    long    undocumented
{63}                    long    undocumented
{64}                    long    undocumented
{65}                    long    undocumented
{66}                    long    get_mem_xy_disp    + (put_h               << 9)                                       '* LD      H,(IX+disp)    DD66disp       H <- (IX+disp)
{67}                    long    undocumented
{68}                    long    undocumented
{69}                    long    undocumented
{6A}                    long    undocumented
{6B}                    long    undocumented
{6C}                    long    undocumented
{6D}                    long    undocumented
{6E}                    long    get_mem_xy_disp    + (put_l               << 9)                                       '* LD      L,(IX+disp)    DD6Edisp       L <- (IX+disp)
{6F}                    long    undocumented

{70}                    long    get_b              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),B    DD70disp       (IX+disp) <- B
{71}                    long    get_c              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),C    DD71disp       (IX+disp) <- C
{72}                    long    get_d              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),D    DD72disp       (IX+disp) <- D
{73}                    long    get_e              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),E    DD73disp       (IX+disp) <- E
{74}                    long    get_h              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),H    DD74disp       (IX+disp) <- H
{75}                    long    get_l              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),L    DD75disp       (IX+disp) <- L
{76}                    long    ld_ix_disp_byte                                                                       'LD      (IX+disp),byte DD76dispbyte   (IX+disp) <- byte
{77}                    long    get_a              + (put_mem_xy_disp     << 9)                                       '* LD      (IX+disp),A    DD77disp       (IX+disp) <- A
{78}                    long    undocumented
{79}                    long    undocumented
{7A}                    long    undocumented
{7B}                    long    undocumented
{7C}                    long    undocumented
{7D}                    long    undocumented
{7E}                    long    get_mem_xy_disp    + (put_a               << 9)                                       '* LD      A,(IX+disp)    DD7Edisp       A <- (IX+disp)
{7F}                    long    undocumented

{80}                    long    undocumented
{81}                    long    undocumented
{82}                    long    undocumented
{83}                    long    undocumented
{84}                    long    undocumented
{85}                    long    undocumented
{86}                    long    get_mem_xy_disp    + (alu_add             << 9) + (put_a          << 18)              '* ADD     A,(IX+disp)    DD86disp       A <- A + (IX+disp)
{87}                    long    undocumented
{88}                    long    undocumented
{89}                    long    undocumented
{8A}                    long    undocumented
{8B}                    long    undocumented
{8C}                    long    undocumented
{8D}                    long    undocumented
{8E}                    long    get_mem_xy_disp    + (alu_add_with_carry  << 9) + (put_a          << 18)              '* ADC     A,(IX+disp)    DD8Edisp       A <- A + (IX+disp) + Carry
{8F}                    long    undocumented

{90}                    long    undocumented
{91}                    long    undocumented
{92}                    long    undocumented
{93}                    long    undocumented
{94}                    long    undocumented
{95}                    long    undocumented
{96}                    long    get_mem_xy_disp    + (alu_sub             << 9) + (put_a          << 18)              '* SUB     (IX+disp)      DD96disp       A <- A - (IX+disp)
{97}                    long    undocumented
{98}                    long    undocumented
{99}                    long    undocumented
{9A}                    long    undocumented
{9B}                    long    undocumented
{9C}                    long    undocumented
{9D}                    long    undocumented
{9E}                    long    get_mem_xy_disp    + (alu_sub_with_borrow << 9) + (put_a          << 18)              '* SBC     (IX+disp)      DD9Edisp       A <- A - (IX+disp) - Carry
{9F}                    long    undocumented

{A0}                    long    undocumented
{A1}                    long    undocumented
{A2}                    long    undocumented
{A3}                    long    undocumented
{A4}                    long    undocumented
{A5}                    long    undocumented
{A6}                    long    get_mem_xy_disp    + (alu_and             << 9) + (put_a          << 18)              '* AND     (IX+disp)      DDA6disp       A <- A AND (IX+disp)
{A7}                    long    undocumented
{A8}                    long    undocumented
{A9}                    long    undocumented
{AA}                    long    undocumented
{AB}                    long    undocumented
{AC}                    long    undocumented
{AD}                    long    undocumented
{AE}                    long    get_mem_xy_disp    + (alu_xor             << 9) + (put_a          << 18)              '* XOR     (IX+disp)      DDAEdisp       A <- A XOR (IX+disp)
{AF}                    long    undocumented

{B0}                    long    undocumented
{B1}                    long    undocumented
{B2}                    long    undocumented
{B3}                    long    undocumented
{B4}                    long    undocumented
{B5}                    long    undocumented
{B6}                    long    get_mem_xy_disp    + (alu_or              << 9) + (put_a          << 18)              '* OR      (IX+disp)      DDB6disp       A <- A OR (IX+disp)
{B7}                    long    undocumented
{B8}                    long    undocumented
{B9}                    long    undocumented
{BA}                    long    undocumented
{BB}                    long    undocumented
{BC}                    long    undocumented
{BD}                    long    undocumented
{BE}                    long    get_mem_xy_disp    + (alu_sub             << 9) + (fetch          << 18)              '* CP      (IX+disp)      DDBEdisp       A - (IX+disp)
{BF}                    long    undocumented

{C0}                    long    undocumented
{C1}                    long    undocumented
{C2}                    long    undocumented
{C3}                    long    undocumented
{C4}                    long    undocumented
{C5}                    long    undocumented
{C6}                    long    undocumented
{C7}                    long    undocumented
{C8}                    long    undocumented
{C9}                    long    undocumented
{CA}                    long    undocumented
{CB}                    long    get_mem_xy_disp    + (z80_ddcb            << 9)                                       'Instruction ddcb prefix
{CC}                    long    undocumented
{CD}                    long    undocumented
{CE}                    long    undocumented
{CF}                    long    undocumented

{D0}                    long    undocumented
{D1}                    long    undocumented
{D2}                    long    undocumented
{D3}                    long    undocumented
{D4}                    long    undocumented
{D5}                    long    undocumented
{D6}                    long    undocumented
{D7}                    long    undocumented
{D8}                    long    undocumented
{D9}                    long    undocumented
{DA}                    long    undocumented
{DB}                    long    undocumented
{DC}                    long    undocumented
{DD}                    long    undocumented
{DE}                    long    undocumented
{DF}                    long    undocumented

{E0}                    long    undocumented
{E1}                    long    pop_               + (put_ixy             << 9)                                       'POP IX     DDE1
{E2}                    long    undocumented
{E3}                    long    not_implemented                                                                       'EX      (SP),IX         DDE3            IXh <-> (SP+1); IXl <-> (SP)
{E4}                    long    undocumented
{E5}                    long    get_ixy            + (push_               << 9)                                       'PUSH    IX              DDE5            (SP-2) <- IXl; (SP-1) <- IXh 
{E6}                    long    undocumented
{E7}                    long    undocumented
{E8}                    long    undocumented
{E9}                    long    get_ixy            + (put_pc              << 9)                                       '* JP      (IX)            DDE9            PC <- IX
{EA}                    long    undocumented
{EB}                    long    undocumented
{EC}                    long    undocumented
{ED}                    long    undocumented
{EE}                    long    undocumented
{EF}                    long    undocumented

{F0}                    long    undocumented
{F1}                    long    undocumented
{F2}                    long    undocumented
{F3}                    long    undocumented
{F4}                    long    undocumented
{F5}                    long    undocumented
{F6}                    long    undocumented
{F7}                    long    undocumented
{F8}                    long    undocumented
{F9}                    long    get_ixy            + (put_sp              << 9)                                       '* LD      SP,IX           DDF9            SP <- IX
{FA}                    long    undocumented
{FB}                    long    undocumented
{FC}                    long    undocumented
{FD}                    long    undocumented
{FE}                    long    undocumented
{FF}                    long    undocumented
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_ddcb
{06}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RLC     (IX+disp)      DDCBdisp06     ---
{0E}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RRC     (IX+disp)      DDCBdisp0E     ---
{16}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RL      (IX+disp)      DDCBdisp16     ---
{1E}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'RL      (IX+disp)      DDCBdisp1E     ---
{26}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'SLA     (IX+disp)      DDCBdisp26     ---
{2E}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'SRA     (IX+disp)      DDCBdisp2E     ---
{36}                    long    undocumented
{3E}                    long    get_stash          + (0                   << 9) + (put_memory_sa  << 18)              'SRL     (IX+disp)      DDCBdisp3E     ---
{46}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (0 << 27)  '*BIT     0,(IX+disp)    DDCBdisp46     Z flag <- NOT 0b
{4E}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (1 << 27)  '*BIT     1,(IX+disp)    DDCBdisp4E     Z flag <- NOT 1b
{56}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (2 << 27)  '*BIT     2,(IX+disp)    DDCBdisp56     Z flag <- NOT 2b
{5E}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (3 << 27)  '*BIT     3,(IX+disp)    DDCBdisp5E     Z flag <- NOT 3b
{66}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (4 << 27)  '*BIT     4,(IX+disp)    DDCBdisp66     Z flag <- NOT 4b
{6E}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (5 << 27)  '*BIT     5,(IX+disp)    DDCBdisp6E     Z flag <- NOT 5b
{76}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (6 << 27)  '*BIT     6,(IX+disp)    DDCBdisp76     Z flag <- NOT 6b
{7E}                    long    get_stash          + (alu_bit             << 9) + (put_memory_sa  << 18) + (7 << 27)  '*BIT     7,(IX+disp)    DDCBdisp7E     Z flag <- NOT 7b
{86}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (0 << 27)  '*RES     0,(IX+disp)    DDCBdisp86     0b <- 0
{8E}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (1 << 27)  '*RES     1,(IX+disp)    DDCBdisp8E     1b <- 0
{96}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (2 << 27)  '*RES     2,(IX+disp)    DDCBdisp96     2b <- 0
{9E}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (3 << 27)  '*RES     3,(IX+disp)    DDCBdisp9E     3b <- 0
{A6}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (4 << 27)  '*RES     4,(IX+disp)    DDCBdispA6     4b <- 0
{AE}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (5 << 27)  '*RES     5,(IX+disp)    DDCBdispAE     5b <- 0
{B6}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (6 << 27)  '*RES     6,(IX+disp)    DDCBdispB6     6b <- 0
{BE}                    long    get_stash          + (alu_res             << 9) + (put_memory_sa  << 18) + (7 << 27)  '*RES     7,(IX+disp)    DDCBdispBE     7b <- 0
{C6}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (0 << 27)  '**SET     0,(IX+disp)    DDCBdispC6     0b <- 1
{CE}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (1 << 27)  '*SET     1,(IX+disp)    DDCBdispCE     1b <- 1
{D6}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (2 << 27)  '*SET     2,(IX+disp)    DDCBdispD6     2b <- 1
{DE}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (3 << 27)  '*SET     3,(IX+disp)    DDCBdispDE     3b <- 1
{E6}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (4 << 27)  '*SET     4,(IX+disp)    DDCBdispE6     4b <- 1
{EE}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (5 << 27)  '*SET     5,(IX+disp)    DDCBdispEE     5b <- 1
{F6}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (6 << 27)  '*SET     6,(IX+disp)    DDCBdispF6     6b <- 1
{FE}                    long    get_stash          + (alu_set             << 9) + (put_memory_sa  << 18) + (7 << 27)  '*SET     7,(IX+disp)    DDCBdispFE     7b <- 1
'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
dispatch_table_ed
{40}                    long    in_c               + (put_b               << 9)                           '*IN      B,(C)           ED40            B <- [C]
{41}                    long    get_b              + (out_c               << 9)                           '*OUT     (C),B           ED41            [C] <- B
{42}                    long    get_bc             + (double_sub_hl_cy    << 9)                           '*SBC     HL,BC           ED42            HL <- HL - BC - Carry
{43}                    long    get_bc             + (store_word_direct   << 9)                           '*LD      (word),BC       ED43word        (word) <- BC
{44}                    long    vector_overlay     + (alu_neg_ovl_no      << 9)                           '*NEG                     ED44            A <- 0-A
{45}                    long    not_implemented                                                           'RETN                    ED45            IFF1 <- IFF2;RETI
{46}                    long    not_implemented                                                           'IM      0               ED46            Set interrupt mode 0
{47}                    long    not_implemented                                                           'LD      I,A             ED47            Interrupt Page <- A
{48}                    long    in_c               + (put_c               << 9)                           '*IN      C,(C)           ED48            C <- [C]
{49}                    long    get_c              + (out_c               << 9)                           '*OUT     (C),C           ED49            [C] <- C
{4A}                    long    get_bc             + (double_add_hl_cy    << 9)                           '*ADC     HL,BC           ED4A            HL <- HL + BC + Carry
{4B}                    long    load_word_direct   + (put_bc              << 9)                           '*LD      BC,(word)       ED4Bword        BC <- (word)
{4C}                    long    undocumented
{4D}                    long    not_implemented                                                           'RETI                    ED4D            Return from Interrupt
{4E}                    long    undocumented
{4F}                    long    not_implemented                                                          'LD       R,A             ED4F           Refresh counter <- A

{50}                    long    in_c               + (put_d               << 9)                           '*IN      D,(C)           ED50            D <- [C]
{51}                    long    get_d              + (out_c               << 9)                           '*OUT     (C),D           ED51            [C] <- D
{52}                    long    get_de             + (double_sub_hl_cy    << 9)                           '*SBC     HL,DE           ED52            HL <- HL - DE - Carry
{53}                    long    get_de             + (store_word_direct   << 9)                           'LD      (word),DE       ED53word        (word) <- DE
{54}                    long    undocumented
{55}                    long    undocumented
{56}                    long    get_mem_xy_disp    + (put_d               << 9)                           'IM      1               ED56            Set interrupt mode 0
{57}                    long    undocumented                                                              'LD       A,I             ED57            A <- Interrupt Page ?? 
{58}                    long    in_c               + (put_e               << 9)                           '*IN      E,(C)           ED58            E <- [C]
{59}                    long    get_e              + (out_c               << 9)                           '*OUT     (C),E           ED59            [C] <- E
{5A}                    long    get_de             + (double_add_hl_cy    << 9)                           '*ADC     HL,DE           ED5A            HL <- HL + DE + Carry
{5B}                    long    load_word_direct   + (put_de              << 9)                           'LD      DE,(word)       ED5Bword        DE <- (word)
{5C}                    long    undocumented
{5D}                    long    undocumented
{5E}                    long    not_implemented                                                           'IM       2               ED5E           Set interrupt mode 0
{5F}                    long    not_implemented                                                           'LD       A,R             ED5F           A <- Refresh counter

{60}                    long    in_c               + (put_h               << 9)                           '*IN      H,(C)           ED60            H <- [C]
{61}                    long    get_h              + (out_c               << 9)                           '*OUT     (C),H           ED61            [C] <- H
{62}                    long    get_hl             + (double_sub_hl_cy    << 9)                           '*SBC     HL,HL           ED62            HL <- HL - HL - Carry
{63}                    long    get_hl             + (store_word_direct   << 9)                           'LD      (word),HL       ED6Bword        (word) <- HL
{64}                    long    undocumented
{65}                    long    undocumented
{66}                    long    undocumented
{67}                    long    not_implemented                                                           'RRD                     ED67            Rotate Right Digit
{68}                    long    in_c               + (put_l               << 9)                           '*IN      L,(C)           ED68            L <- [C]
{69}                    long    get_l              + (out_c               << 9)                           '*OUT     (C),L           ED69            [C] <- L
{6A}                    long    get_hl             + (double_add_hl_cy    << 9)                           '*ADC     HL,HL           ED6A            HL <- HL + HL + Carry
{6B}                    long    load_word_direct   + (put_hl              << 9)                           '*LD      HL,(word)       ED6Bword        HL <- (word)
{6C}                    long    undocumented
{6D}                    long    undocumented
{6E}                    long    undocumented
{6F}                    long    not_implemented                                                           'RLD                     ED6F            Rotate Left Digit

{70}                    long    undocumented
{71}                    long    undocumented
{72}                    long    get_sp             + (double_sub_hl_cy    << 9)                           '*SBC     HL,SP           ED72            HL <- HL - SP - Carry
{73}                    long    get_sp             + (store_word_direct   << 9)                           'LD      (word),SP       ED73word        (word) <- SP
{74}                    long    undocumented
{75}                    long    undocumented
{76}                    long    undocumented
{77}                    long    undocumented
{78}                    long    in_c               + (put_a               << 9)                           '*IN      A,(C)           ED78            A <- [C]
{79}                    long    get_a              + (out_c               << 9)                           '*OUT     (C),A           ED79            [C] <- A
{7A}                    long    get_sp             + (double_add_hl_cy    << 9)                           '*ADC     HL,SP           ED7A            HL <- HL + SP + Carry
{7B}                    long    load_word_direct   + (put_sp              << 9)                           'LD      SP,(word)        ED7Bword        SP <- (word)
{7C}                    long    undocumented
{7D}                    long    undocumented
{7E}                    long    undocumented
{7F}                    long    undocumented

{80}                    long    undocumented
{81}                    long    undocumented
{82}                    long    undocumented
{83}                    long    undocumented
{84}                    long    undocumented
{85}                    long    undocumented
{86}                    long    undocumented
{87}                    long    undocumented
{88}                    long    undocumented
{89}                    long    undocumented
{8A}                    long    undocumented
{8B}                    long    undocumented
{8C}                    long    undocumented
{8D}                    long    undocumented
{8E}                    long    undocumented
{8F}                    long    undocumented

{90}                    long    undocumented
{91}                    long    undocumented
{92}                    long    undocumented
{93}                    long    undocumented
{94}                    long    undocumented
{95}                    long    undocumented
{96}                    long    undocumented
{97}                    long    undocumented
{98}                    long    undocumented
{99}                    long    undocumented
{9A}                    long    undocumented
{9B}                    long    undocumented
{9C}                    long    undocumented
{9D}                    long    undocumented
{9E}                    long    undocumented
{9F}                    long    undocumented

{A0}                    long    vector_overlay     + (ldi_ovl_no          << 9)                           'LDI                     EDA0            (DE) <- (HL); HL <- HL+1; DE <- DE+1; BC <- BC-1
{A1}                    long    vector_overlay     + (cpi_ovl_no          << 9)                           'CPI                     EDA1            A - (HL); HL <- HL+1; BC <- BC-1
{A2}                    long    vector_overlay     + (ini_ovl_no          << 9)                           'INI                     EDA2            (HL) <- [C]; B <- B-1; HL <- HL+1
{A3}                    long    vector_overlay     + (outi_ovl_no         << 9)                           'OUTI                    EDA3            [C] <- (HL); B <- B-1; HL <- HL+1
{A4}                    long    undocumented
{A5}                    long    undocumented
{A6}                    long    undocumented
{A7}                    long    undocumented
{A8}                    long    vector_overlay     + (ldd_ovl_no          << 9)                           'LDD                     EDA8            (DE) <- (HL); HL <- HL-1; DE <- DE-1; BC <- BC-1
{A9}                    long    vector_overlay     + (cpd_ovl_no          << 9)                           'CPD                     EDA9            A - (HL); HL <- HL-1; BC <- BC-1
{AA}                    long    vector_overlay     + (ind_ovl_no          << 9)                           'IND                     EDAA            (HL) <- [C]; B <- B-1; HL <- HL-1
{AB}                    long    vector_overlay     + (outd_ovl_no         << 9)                           'OUTD                    EDAB            [C] <- (HL); B <- B-1; HL <- HL-1
{AC}                    long    undocumented
{AD}                    long    undocumented
{AE}                    long    undocumented
{AF}                    long    undocumented

{B0}                    long    vector_overlay     + (ldir_ovl_no         << 9)                           'LDIR                    EDB0            (DE) <- (HL); HL <- HL+1; DE <- DE+1; BC <- BC-1
{B1}                    long    vector_overlay     + (cpir_ovl_no         << 9)                           'CPIR                    EDB1            A - (HL); HL <- HL+1; BC <- BC-1
{B2}                    long    vector_overlay     + (inir_ovl_no         << 9)                           'INIR                    EDB2            (HL) <- [C]; B <- B-1; HL <- HL+1
{B3}                    long    vector_overlay     + (otir_ovl_no         << 9)                           'OTIR                    EDB3            [C] <- (HL); B <- B-1; HL <- HL+1
{B4}                    long    undocumented
{B5}                    long    undocumented
{B6}                    long    undocumented
{B7}                    long    undocumented
{B8}                    long    vector_overlay     + (lddr_ovl_no         << 9)                           'LDDR                    EDB8            (DE) <- (HL); HL <- HL-1; DE <- DE-1; BC <- BC-1
{B9}                    long    vector_overlay     + (cpdr_ovl_no         << 9)                           'CPDR                    EDB9            A - (HL); HL <- HL-1; BC <- BC-1
{BA}                    long    vector_overlay     + (indr_ovl_no         << 9)                           'INDR                    EDBA            (HL) <- [C]; B <- B-1; HL <- HL-1
{BB}                    long    vector_overlay     + (otdr_ovl_no         << 9)                           'OTDR                    EDBB            [C] <- (HL); B <- B-1; HL <- HL-1
'---------------------------------------------------------------------------------------------------------

' The end.